import logging
import multiprocessing
import sys

from mopidy import settings, SettingsError
from mopidy.utils import get_class, unpickle_connection

logger = logging.getLogger('mopidy.process')

class BaseProcess(multiprocessing.Process):
    def run(self):
        try:
            self._run()
        except KeyboardInterrupt:
            logger.info(u'Interrupted by user')
            sys.exit(0)
        except SettingsError as e:
            logger.error(e.message)
            sys.exit(1)

    def _run(self):
        raise NotImplementedError


class CoreProcess(BaseProcess):
    def __init__(self, core_queue):
        super(CoreProcess, self).__init__()
        self.core_queue = core_queue
        self._backend = None
        self._frontend = None

    def _run(self):
        self._setup()
        while True:
            message = self.core_queue.get()
            self._process_message(message)

    def _setup(self):
        self._backend = get_class(settings.BACKENDS[0])(
            core_queue=self.core_queue)
        self._frontend = get_class(settings.FRONTEND)(backend=self._backend)

    def _process_message(self, message):
        if message['command'] == 'mpd_request':
            response = self._frontend.handle_request(message['request'])
            connection = unpickle_connection(message['reply_to'])
            connection.send(response)
        elif message['command'] == 'end_of_track':
            self._backend.playback.end_of_track_callback()
        elif message['command'] == 'stop_playback':
            self._backend.playback.stop()
        elif message['command'] == 'set_stored_playlists':
            self._backend.stored_playlists.playlists = message['playlists']
        else:
            logger.warning(u'Cannot handle message: %s', message)



from mopidy.backends import (BaseBackend, BaseCurrentPlaylistController,
    BasePlaybackController, BaseLibraryController,
    BaseStoredPlaylistsController)
from mopidy.models import Playlist

class DummyBackend(BaseBackend):
    """
    A backend which implements the backend API in the simplest way possible.
    Used in tests of the frontends.

    Handles URIs starting with ``dummy:``.
    """

    def __init__(self, *args, **kwargs):
        super(DummyBackend, self).__init__(*args, **kwargs)
        self.current_playlist = DummyCurrentPlaylistController(backend=self)
        self.library = DummyLibraryController(backend=self)
        self.playback = DummyPlaybackController(backend=self)
        self.stored_playlists = DummyStoredPlaylistsController(backend=self)
        self.uri_handlers = [u'dummy:']

class DummyCurrentPlaylistController(BaseCurrentPlaylistController):
    pass

class DummyLibraryController(BaseLibraryController):
    _library = []

    def lookup(self, uri):
        matches = filter(lambda t: uri == t.uri, self._library)
        if matches:
            return matches[0]

    def search(self, field, query):
        return Playlist()

    find_exact = search

class DummyPlaybackController(BasePlaybackController):
    def _next(self, track):
        return True

    def _pause(self):
        return True

    def _play(self, track):
        return True

    def _previous(self, track):
        return True

    def _resume(self):
        return True

class DummyStoredPlaylistsController(BaseStoredPlaylistsController):
    def search(self, query):
        return [Playlist(name=query)]



"""
Available settings and their default values.

.. warning::

    Do *not* change settings in ``mopidy/settings.py``. Instead, add a file
    called ``~/.mopidy/settings.py`` and redefine settings there.
"""

from __future__ import absolute_import
import os
import sys

#: List of playback backends to use. See :mod:`mopidy.backends` for all
#: available backends. Default::
#:
#:     BACKENDS = (u'mopidy.backends.despotify.DespotifyBackend',)
#:
#: .. note::
#:     Currently only the first backend in the list is used.
BACKENDS = (
    u'mopidy.backends.despotify.DespotifyBackend',
    #u'mopidy.backends.libspotify.LibspotifyBackend',
)

#: The log format used on the console. See
#: http://docs.python.org/library/logging.html#formatter-objects for details on
#: the format.
CONSOLE_LOG_FORMAT = u'%(levelname)-8s %(asctime)s' + \
    ' [%(process)d:%(threadName)s] %(name)s\n  %(message)s'

#: The log format used for dump logs. Default::
#:
#:     DUMP_LOG_FILENAME = CONSOLE_LOG_FORMAT
DUMP_LOG_FORMAT = CONSOLE_LOG_FORMAT

#: The file to dump debug log data to. Default::
#:
#:     DUMP_LOG_FILENAME = u'dump.log'
DUMP_LOG_FILENAME = u'dump.log'

#: Protocol frontend to use. Default::
#:
#:     FRONTEND = u'mopidy.mpd.frontend.MpdFrontend'
FRONTEND = u'mopidy.mpd.frontend.MpdFrontend'

#: Path to folder with local music. Default::
#:
#:    LOCAL_MUSIC_FOLDER = u'~/music'
LOCAL_MUSIC_FOLDER = u'~/music'

#: Path to playlist folder with m3u files for local music. Default::
#:
#:    LOCAL_PLAYLIST_FOLDER = u'~/.mopidy/playlists'
LOCAL_PLAYLIST_FOLDER = u'~/.mopidy/playlists'

#: Path to tag cache for local music. Default::
#:
#:    LOCAL_TAG_CACHE = u'~/.mopidy/tag_cache'
LOCAL_TAG_CACHE = u'~/.mopidy/tag_cache'

#: Sound mixer to use. See :mod:`mopidy.mixers` for all available mixers.
#:
#: Default on Linux::
#:
#:     MIXER = u'mopidy.mixers.alsa.AlsaMixer'
#:
#: Default on OS X::
#:
#:     MIXER = u'mopidy.mixers.osa.OsaMixer'
#:
#: Default on other operating systems::
#:
#:     MIXER = u'mopidy.mixers.dummy.DummyMixer'
MIXER = u'mopidy.mixers.dummy.DummyMixer'
if sys.platform == 'linux2':
    MIXER = u'mopidy.mixers.alsa.AlsaMixer'
elif sys.platform == 'darwin':
    MIXER = u'mopidy.mixers.osa.OsaMixer'

#: ALSA mixer only. What mixer control to use. If set to :class:`False`, first
#: ``Master`` and then ``PCM`` will be tried.
#:
#: Example: ``Master Front``. Default: :class:`False`
MIXER_ALSA_CONTROL = False

#: External mixers only. Which port the mixer is connected to.
#:
#: This must point to the device port like ``/dev/ttyUSB0``.
#: Default: :class:`None`
MIXER_EXT_PORT = None

#: External mixers only. What input source the external mixer should use.
#:
#: Example: ``Aux``. Default: :class:`None`
MIXER_EXT_SOURCE = None

#: External mixers only. What state Speakers A should be in.
#:
#: Default: :class:`None`.
MIXER_EXT_SPEAKERS_A = None

#: External mixers only. What state Speakers B should be in.
#:
#: Default: :class:`None`.
MIXER_EXT_SPEAKERS_B = None

#: Server to use. Default::
#:
#:     SERVER = u'mopidy.mpd.server.MpdServer'
SERVER = u'mopidy.mpd.server.MpdServer'

#: Which address Mopidy should bind to. Examples:
#:
#: ``127.0.0.1``
#:     Listens only on the IPv4 loopback interface. Default.
#: ``::1``
#:     Listens only on the IPv6 loopback interface.
#: ``0.0.0.0``
#:     Listens on all IPv4 interfaces.
#: ``::``
#:     Listens on all interfaces, both IPv4 and IPv6.
SERVER_HOSTNAME = u'127.0.0.1'

#: Which TCP port Mopidy should listen to. Default: 6600
SERVER_PORT = 6600

#: Your Spotify Premium username. Used by all Spotify backends.
SPOTIFY_USERNAME = u''

#: Your Spotify Premium password. Used by all Spotify backends.
SPOTIFY_PASSWORD = u''

#: Path to your libspotify application key. Used by LibspotifyBackend.
SPOTIFY_LIB_APPKEY = u'~/.mopidy/spotify_appkey.key'

#: Path to the libspotify cache. Used by LibspotifyBackend.
SPOTIFY_LIB_CACHE = u'~/.mopidy/libspotify_cache'

# Import user specific settings
dotdir = os.path.expanduser(u'~/.mopidy/')
settings_file = os.path.join(dotdir, u'settings.py')
if os.path.isfile(settings_file):
    sys.path.insert(0, dotdir)
    from settings import *



class BaseMixer(object):
    def __init__(self, *args, **kwargs):
        pass

    @property
    def volume(self):
        """
        The audio volume

        Integer in range [0, 100]. :class:`None` if unknown. Values below 0 is
        equal to 0. Values above 100 is equal to 100.
        """
        return self._get_volume()

    @volume.setter
    def volume(self, volume):
        volume = int(volume)
        if volume < 0:
            volume = 0
        elif volume > 100:
            volume = 100
        self._set_volume(volume)

    def destroy(self):
        pass

    def _get_volume(self):
        """
        Return volume as integer in range [0, 100]. :class:`None` if unknown.

        *Must be implemented by subclass.*
        """
        raise NotImplementedError

    def _set_volume(self, volume):
        """
        Set volume as integer in range [0, 100].

        *Must be implemented by subclass.*
        """
        raise NotImplementedError



from mopidy import MopidyException

class MpdAckError(MopidyException):
    """
    Available MPD error codes::

        ACK_ERROR_NOT_LIST = 1
        ACK_ERROR_ARG = 2
        ACK_ERROR_PASSWORD = 3
        ACK_ERROR_PERMISSION = 4
        ACK_ERROR_UNKNOWN = 5
        ACK_ERROR_NO_EXIST = 50
        ACK_ERROR_PLAYLIST_MAX = 51
        ACK_ERROR_SYSTEM = 52
        ACK_ERROR_PLAYLIST_LOAD = 53
        ACK_ERROR_UPDATE_ALREADY = 54
        ACK_ERROR_PLAYER_SYNC = 55
        ACK_ERROR_EXIST = 56
    """

    def __init__(self, message=u'', error_code=0, index=0, command=u''):
        self.message = message
        self.error_code = error_code
        self.index = index
        self.command = command

    def get_mpd_ack(self):
        """
        MPD error code format::

            ACK [%(error_code)i@%(index)i] {%(command)s} description
        """
        return u'ACK [%i@%i] {%s} %s' % (
            self.error_code, self.index, self.command, self.message)

class MpdArgError(MpdAckError):
    def __init__(self, *args, **kwargs):
        super(MpdArgError, self).__init__(*args, **kwargs)
        self.error_code = 2 # ACK_ERROR_ARG

class MpdUnknownCommand(MpdAckError):
    def __init__(self, *args, **kwargs):
        super(MpdUnknownCommand, self).__init__(*args, **kwargs)
        self.message = u'unknown command "%s"' % self.command
        self.command = u''
        self.error_code = 5 # ACK_ERROR_UNKNOWN

class MpdNoExistError(MpdAckError):
    def __init__(self, *args, **kwargs):
        super(MpdNoExistError, self).__init__(*args, **kwargs)
        self.error_code = 50 # ACK_ERROR_NO_EXIST

class MpdNotImplemented(MpdAckError):
    def __init__(self, *args, **kwargs):
        super(MpdNotImplemented, self).__init__(*args, **kwargs)
        self.message = u'Not implemented'



import logging
from multiprocessing.reduction import reduce_connection
import os
import pickle
import sys
import urllib

logger = logging.getLogger('mopidy.utils')

from mopidy.models import Track, Artist, Album

def flatten(the_list):
    result = []
    for element in the_list:
        if isinstance(element, list):
            result.extend(flatten(element))
        else:
            result.append(element)
    return result

def get_class(name):
    module_name = name[:name.rindex('.')]
    class_name = name[name.rindex('.') + 1:]
    logger.debug('Loading: %s', name)
    module = __import__(module_name, globals(), locals(), [class_name], -1)
    class_object = getattr(module, class_name)
    return class_object

def get_or_create_folder(folder):
    folder = os.path.expanduser(folder)
    if not os.path.isdir(folder):
        logger.info(u'Creating %s', folder)
        os.mkdir(folder, 0755)
    return folder

def path_to_uri(*paths):
    path = os.path.join(*paths)
    #path = os.path.expanduser(path) # FIXME
    path = path.encode('utf-8')
    if sys.platform == 'win32':
        return 'file:' + urllib.pathname2url(path)
    return 'file://' + urllib.pathname2url(path)

def indent(string, places=4, linebreak='\n'):
    lines = string.split(linebreak)
    if len(lines) == 1:
        return string
    result = u''
    for line in lines:
        result += linebreak + ' ' * places + line
    return result

def pickle_connection(connection):
    return pickle.dumps(reduce_connection(connection))

def unpickle_connection(pickled_connection):
    # From http://stackoverflow.com/questions/1446004
    (func, args) = pickle.loads(pickled_connection)
    return func(*args)

def spotify_uri_to_int(uri, output_bits=31):
    """
    Stable one-way translation from Spotify URI to 31-bit integer.

    Spotify track URIs has 62^22 possible values, which requires 131 bits of
    storage. The original MPD server uses 32-bit unsigned integers for track
    IDs. GMPC seems to think the track ID is a signed integer, thus we use 31
    output bits.

    In other words, this function throws away 100 bits of information. Since we
    only use the track IDs to identify a track within a single Mopidy instance,
    this information loss is acceptable. The chances of getting two different
    tracks with the same track ID loaded in the same Mopidy instance is still
    rather slim. 1 to 2,147,483,648 to be exact.

    Normal usage, with data loss::

        >>> spotify_uri_to_int('spotify:track:5KRRcT67VNIZUygEbMoIC1')
        624351954

    No data loss, may be converted back into a Spotify URI::

        >>> spotify_uri_to_int('spotify:track:5KRRcT67VNIZUygEbMoIC1',
        ...     output_bits=131)
        101411513484007705241035418492696638725L

    :param uri: Spotify URI on the format ``spotify:track:*``
    :type uri: string
    :param output_bits: number of bits of information kept in the return value
    :type output_bits: int
    :rtype: int
    """

    CHARS = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
    BITS_PER_CHAR = 6 # int(math.ceil(math.log(len(CHARS), 2)))

    key = uri.split(':')[-1]
    full_id = 0
    for i, char in enumerate(key):
        full_id ^= CHARS.index(char) << BITS_PER_CHAR * i
    compressed_id = 0
    while full_id != 0:
        compressed_id ^= (full_id & (2 ** output_bits - 1))
        full_id >>= output_bits
    return int(compressed_id)

def parse_m3u(file_path):
    """
    Convert M3U file list of uris

    Example M3U data::

        # This is a comment
        Alternative\Band - Song.mp3
        Classical\Other Band - New Song.mp3
        Stuff.mp3
        D:\More Music\Foo.mp3
        http://www.example.com:8000/Listen.pls
        http://www.example.com/~user/Mine.mp3

    - Relative paths of songs should be with respect to location of M3U.
    - Paths are normaly platform specific.
    - Lines starting with # should be ignored.
    - m3u files are latin-1.
    - This function does not bother with Extended M3U directives.
    """

    uris = []
    folder = os.path.dirname(file_path)

    try:
        with open(file_path) as m3u:
            contents = m3u.readlines()
    except IOError, e:
        logger.error('Couldn\'t open m3u: %s', e)
        return uris

    for line in contents:
        line = line.strip().decode('latin1')

        if line.startswith('#'):
            continue

        # FIXME what about other URI types?
        if line.startswith('file://'):
            uris.append(line)
        else:
            path = path_to_uri(folder, line)
            uris.append(path)

    return uris

def parse_mpd_tag_cache(tag_cache, music_dir=''):
    """
    Converts a MPD tag_cache into a lists of tracks, artists and albums.
    """
    tracks = set()

    try:
        with open(tag_cache) as library:
            contents = library.read()
    except IOError, e:
        logger.error('Could not open tag cache: %s', e)
        return tracks

    current = {}
    state = None

    for line in contents.split('\n'):
        if line == 'songList begin':
            state = 'songs'
            continue
        elif line == 'songList end':
            state = None
            continue
        elif not state:
            continue

        key, value = line.split(': ', 1)

        if key == 'key':
            _convert_mpd_data(current, tracks, music_dir)
            current.clear()

        current[key.lower()] = value

    _convert_mpd_data(current, tracks, music_dir)

    return tracks

def _convert_mpd_data(data, tracks, music_dir):
    if not data:
        return

    track_kwargs = {}
    album_kwargs = {}

    if 'track' in data:
        album_kwargs['num_tracks'] = int(data['track'].split('/')[1])
        track_kwargs['track_no'] = int(data['track'].split('/')[0])

    if 'artist' in data:
        artist = Artist(name=data['artist'])
        track_kwargs['artists'] = [artist]
        album_kwargs['artists'] = [artist]

    if 'album' in data:
        album_kwargs['name'] = data['album']
        album = Album(**album_kwargs)
        track_kwargs['album'] = album

    if 'title' in data:
        track_kwargs['name'] = data['title']

    if data['file'][0] == '/':
        path = data['file'][1:]
    else:
        path = data['file']

    track_kwargs['uri'] = path_to_uri(music_dir, path)
    track_kwargs['length'] = int(data.get('time', 0)) * 1000
    track_kwargs['id'] = len(tracks)

    track = Track(**track_kwargs)
    tracks.add(track)



"""
This is our MPD server implementation.
"""

import asynchat
import asyncore
import logging
import multiprocessing
import re
import socket
import sys

from mopidy import get_mpd_protocol_version, settings
from mopidy.utils import indent, pickle_connection

logger = logging.getLogger('mopidy.mpd.server')

#: The MPD protocol uses UTF-8 for encoding all data.
ENCODING = u'utf-8'

#: The MPD protocol uses ``\n`` as line terminator.
LINE_TERMINATOR = u'\n'

class MpdServer(asyncore.dispatcher):
    """
    The MPD server. Creates a :class:`MpdSession` for each client connection.
    """

    def __init__(self, core_queue):
        asyncore.dispatcher.__init__(self)
        self.core_queue = core_queue

    def start(self):
        try:
            if socket.has_ipv6:
                self.create_socket(socket.AF_INET6, socket.SOCK_STREAM)
            else:
                self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
            self.set_reuse_addr()
            hostname = self._format_hostname(settings.SERVER_HOSTNAME)
            port = settings.SERVER_PORT
            logger.debug(u'Binding to [%s]:%s', hostname, port)
            self.bind((hostname, port))
            self.listen(1)
            logger.info(u'MPD server running at [%s]:%s',
                self._format_hostname(settings.SERVER_HOSTNAME),
                settings.SERVER_PORT)
        except IOError, e:
            sys.exit('MPD server startup failed: %s' % e)

    def handle_accept(self):
        (client_socket, client_socket_address) = self.accept()
        logger.info(u'MPD client connection from [%s]:%s',
            client_socket_address[0], client_socket_address[1])
        MpdSession(self, client_socket, client_socket_address,
            self.core_queue).start()

    def handle_close(self):
        self.close()

    def _format_hostname(self, hostname):
        if (socket.has_ipv6
            and re.match('\d+.\d+.\d+.\d+', hostname) is not None):
            hostname = '::ffff:%s' % hostname
        return hostname


class MpdSession(asynchat.async_chat):
    """
    The MPD client session. Dispatches MPD requests to the frontend.
    """

    def __init__(self, server, client_socket, client_socket_address,
            core_queue):
        asynchat.async_chat.__init__(self, sock=client_socket)
        self.server = server
        self.client_address = client_socket_address[0]
        self.client_port = client_socket_address[1]
        self.core_queue = core_queue
        self.input_buffer = []
        self.set_terminator(LINE_TERMINATOR.encode(ENCODING))

    def start(self):
        self.send_response(u'OK MPD %s' % get_mpd_protocol_version())

    def collect_incoming_data(self, data):
        self.input_buffer.append(data)

    def found_terminator(self):
        data = ''.join(self.input_buffer).strip()
        self.input_buffer = []
        try:
            request = data.decode(ENCODING)
            logger.debug(u'Input from [%s]:%s: %s', self.client_address,
                self.client_port, indent(request))
            self.handle_request(request)
        except UnicodeDecodeError as e:
            logger.warning(u'Received invalid data: %s', e)

    def handle_request(self, request):
        my_end, other_end = multiprocessing.Pipe()
        self.core_queue.put({
            'command': 'mpd_request',
            'request': request,
            'reply_to': pickle_connection(other_end),
        })
        my_end.poll(None)
        response = my_end.recv()
        if response is not None:
            self.handle_response(response)

    def handle_response(self, response):
        self.send_response(LINE_TERMINATOR.join(response))

    def send_response(self, output):
        logger.debug(u'Output to [%s]:%s: %s', self.client_address,
            self.client_port, indent(output))
        output = u'%s%s' % (output, LINE_TERMINATOR)
        data = output.encode(ENCODING)
        self.push(data)



from copy import copy

from mopidy.mpd import serializer

class ImmutableObject(object):
    """
    Superclass for immutable objects whose fields can only be modified via the
    constructor.

    :param kwargs: kwargs to set as fields on the object
    :type kwargs: any
    """

    def __init__(self, *args, **kwargs):
        for key, value in kwargs.items():
            if not hasattr(self, key):
                raise TypeError('__init__() got an unexpected keyword ' + \
                    'argument \'%s\'' % key)
            self.__dict__[key] = value

    def __setattr__(self, name, value):
        if name.startswith('_'):
            return super(ImmutableObject, self).__setattr__(name, value)
        raise AttributeError('Object is immutable.')

    def __hash__(self):
        hash_sum = 0
        for key, value in self.__dict__.items():
            hash_sum += hash(key) + hash(value)
        return hash_sum

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)


class Artist(ImmutableObject):
    """
    :param uri: artist URI
    :type uri: string
    :param name: artist name
    :type name: string
    """

    #: The artist URI. Read-only.
    uri = None

    #: The artist name. Read-only.
    name = None


class Album(ImmutableObject):
    """
    :param uri: album URI
    :type uri: string
    :param name: album name
    :type name: string
    :param artists: album artists
    :type artists: list of :class:`Artist`
    :param num_tracks: number of tracks in album
    :type num_tracks: integer
    """

    #: The album URI. Read-only.
    uri = None

    #: The album name. Read-only.
    name = None

    #: The number of tracks in the album. Read-only.
    num_tracks = 0

    def __init__(self, *args, **kwargs):
        self._artists = frozenset(kwargs.pop('artists', []))
        super(Album, self).__init__(*args, **kwargs)

    @property
    def artists(self):
        """List of :class:`Artist` elements. Read-only."""
        return list(self._artists)


class Track(ImmutableObject):
    """
    :param uri: track URI
    :type uri: string
    :param name: track name
    :type name: string
    :param artists: track artists
    :type artists: list of :class:`Artist`
    :param album: track album
    :type album: :class:`Album`
    :param track_no: track number in album
    :type track_no: integer
    :param date: track release date
    :type date: :class:`datetime.date`
    :param length: track length in milliseconds
    :type length: integer
    :param bitrate: bitrate in kbit/s
    :type bitrate: integer
    :param id: track ID (unique and non-changing as long as the process lives)
    :type id: integer
    """

    #: The track URI. Read-only.
    uri = None

    #: The track name. Read-only.
    name = None

    #: The track :class:`Album`. Read-only.
    album = None

    #: The track number in album. Read-only.
    track_no = 0

    #: The track release date. Read-only.
    date = None

    #: The track length in milliseconds. Read-only.
    length = None

    #: The track's bitrate in kbit/s. Read-only.
    bitrate = None

    #: The track ID. Read-only.
    id = None

    def __init__(self, *args, **kwargs):
        self._artists = frozenset(kwargs.pop('artists', []))
        super(Track, self).__init__(*args, **kwargs)

    @property
    def artists(self):
        """List of :class:`Artist`. Read-only."""
        return list(self._artists)

    def mpd_format(self, *args, **kwargs):
        return serializer.track_to_mpd_format(self, *args, **kwargs)


class Playlist(ImmutableObject):
    """
        :param uri: playlist URI
        :type uri: string
        :param name: playlist name
        :type name: string
        :param tracks: playlist's tracks
        :type tracks: list of :class:`Track` elements
        :param last_modified: playlist's modification time
        :type last_modified: :class:`datetime.datetime`
    """

    #: The playlist URI. Read-only.
    uri = None

    #: The playlist name. Read-only.
    name = None

    #: The playlist modification time. Read-only.
    #:
    #: :class:`datetime.datetime`, or :class:`None` if unknown.
    last_modified = None

    def __init__(self, *args, **kwargs):
        self._tracks = kwargs.pop('tracks', [])
        super(Playlist, self).__init__(*args, **kwargs)

    @property
    def tracks(self):
        """List of :class:`Track` elements. Read-only."""
        return copy(self._tracks)

    @property
    def length(self):
        """The number of tracks in the playlist. Read-only."""
        return len(self._tracks)

    def mpd_format(self, *args, **kwargs):
        return serializer.playlist_to_mpd_format(self, *args, **kwargs)

    def with_(self, uri=None, name=None, tracks=None, last_modified=None):
        """
        Create a new playlist object with the given values. The values that are
        not given are taken from the object the method is called on.

        Does not change the object on which it is called.

        :param uri: playlist URI
        :type uri: string
        :param name: playlist name
        :type name: string
        :param tracks: playlist's tracks
        :type tracks: list of :class:`Track` elements
        :param last_modified: playlist's modification time
        :type last_modified: :class:`datetime.datetime`
        :rtype: :class:`Playlist`
        """
        if uri is None:
            uri = self.uri
        if name is None:
            name = self.name
        if tracks is None:
            tracks = self.tracks
        if last_modified is None:
            last_modified = self.last_modified
        return Playlist(uri=uri, name=name, tracks=tracks,
            last_modified=last_modified)



import asyncore
import logging
import logging.handlers
import multiprocessing
import optparse
import os
import sys

sys.path.insert(0,
    os.path.abspath(os.path.join(os.path.dirname(__file__), '../')))

from mopidy import get_version, settings, SettingsError
from mopidy.process import CoreProcess
from mopidy.utils import get_class, get_or_create_folder

logger = logging.getLogger('mopidy.main')

def main():
    options = _parse_options()
    _setup_logging(options.verbosity_level, options.dump)
    logger.info('-- Starting Mopidy --')
    get_or_create_folder('~/.mopidy/')
    core_queue = multiprocessing.Queue()
    get_class(settings.SERVER)(core_queue).start()
    core = CoreProcess(core_queue)
    core.start()
    asyncore.loop()

def _parse_options():
    parser = optparse.OptionParser(version='Mopidy %s' % get_version())
    parser.add_option('-q', '--quiet',
        action='store_const', const=0, dest='verbosity_level',
        help='less output (warning level)')
    parser.add_option('-v', '--verbose',
        action='store_const', const=2, dest='verbosity_level',
        help='more output (debug level)')
    parser.add_option('--dump',
        action='store_true', dest='dump',
        help='dump debug log to file')
    return parser.parse_args()[0]

def _setup_logging(verbosity_level, dump):
    _setup_console_logging(verbosity_level)
    if dump:
        _setup_dump_logging()

def _setup_console_logging(verbosity_level):
    if verbosity_level == 0:
        level = logging.WARNING
    elif verbosity_level == 2:
        level = logging.DEBUG
    else:
        level = logging.INFO
    logging.basicConfig(format=settings.CONSOLE_LOG_FORMAT, level=level)

def _setup_dump_logging():
    root = logging.getLogger('')
    root.setLevel(logging.DEBUG)
    formatter = logging.Formatter(settings.DUMP_LOG_FORMAT)
    handler = logging.handlers.RotatingFileHandler(
        settings.DUMP_LOG_FILENAME, maxBytes=102400, backupCount=3)
    handler.setFormatter(formatter)
    root.addHandler(handler)

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        logger.info(u'Interrupted by user')
        sys.exit(0)
    except SettingsError, e:
        logger.error(e)
        sys.exit(1)
    except SystemExit, e:
        logger.error(e)
        sys.exit(1)



import datetime as dt
import logging
import sys

import spytify

from mopidy import settings
from mopidy.backends import (BaseBackend, BaseCurrentPlaylistController,
    BaseLibraryController, BasePlaybackController,
    BaseStoredPlaylistsController)
from mopidy.models import Artist, Album, Track, Playlist
from mopidy.utils import spotify_uri_to_int

logger = logging.getLogger('mopidy.backends.despotify')

ENCODING = 'utf-8'

class DespotifyBackend(BaseBackend):
    """
    A Spotify backend which uses the open source `despotify library
    <http://despotify.se/>`_.

    `spytify <http://despotify.svn.sourceforge.net/viewvc/despotify/src/bindings/python/>`_
    is the Python bindings for the despotify library. It got litle
    documentation, but a couple of examples are available.

    **Issues:** http://github.com/jodal/mopidy/issues/labels/backend-despotify
    """

    def __init__(self, *args, **kwargs):
        super(DespotifyBackend, self).__init__(*args, **kwargs)
        self.current_playlist = DespotifyCurrentPlaylistController(backend=self)
        self.library = DespotifyLibraryController(backend=self)
        self.playback = DespotifyPlaybackController(backend=self)
        self.stored_playlists = DespotifyStoredPlaylistsController(backend=self)
        self.uri_handlers = [u'spotify:', u'http://open.spotify.com/']
        self.spotify = self._connect()
        self.stored_playlists.refresh()

    def _connect(self):
        logger.info(u'Connecting to Spotify')
        try:
            return DespotifySessionManager(
                settings.SPOTIFY_USERNAME.encode(ENCODING),
                settings.SPOTIFY_PASSWORD.encode(ENCODING),
                core_queue=self.core_queue)
        except spytify.SpytifyError as e:
            logger.exception(e)
            sys.exit(1)


class DespotifyCurrentPlaylistController(BaseCurrentPlaylistController):
    pass


class DespotifyLibraryController(BaseLibraryController):
    def lookup(self, uri):
        track = self.backend.spotify.lookup(uri.encode(ENCODING))
        return DespotifyTranslator.to_mopidy_track(track)

    def search(self, field, what):
        if field == u'track':
            field = u'title'
        if field == u'any':
            query = what
        else:
            query = u'%s:%s' % (field, what)
        result = self.backend.spotify.search(query.encode(ENCODING))
        if (result is None or result.playlist.tracks[0].get_uri() ==
                'spotify:track:0000000000000000000000'):
            return Playlist()
        return DespotifyTranslator.to_mopidy_playlist(result.playlist)

    find_exact = search


class DespotifyPlaybackController(BasePlaybackController):
    def _pause(self):
        try:
            self.backend.spotify.pause()
            return True
        except spytify.SpytifyError as e:
            logger.error(e)
            return False

    def _play(self, track):
        try:
            self.backend.spotify.play(self.backend.spotify.lookup(track.uri))
            return True
        except spytify.SpytifyError as e:
            logger.error(e)
            return False

    def _resume(self):
        try:
            self.backend.spotify.resume()
            return True
        except spytify.SpytifyError as e:
            logger.error(e)
            return False

    def _stop(self):
        try:
            self.backend.spotify.stop()
            return True
        except spytify.SpytifyError as e:
            logger.error(e)
            return False


class DespotifyStoredPlaylistsController(BaseStoredPlaylistsController):
    def refresh(self):
        logger.info(u'Caching stored playlists')
        playlists = []
        for spotify_playlist in self.backend.spotify.stored_playlists:
            playlists.append(
                DespotifyTranslator.to_mopidy_playlist(spotify_playlist))
        self._playlists = playlists
        logger.debug(u'Available playlists: %s',
            u', '.join([u'<%s>' % p.name for p in self.playlists]))
        logger.info(u'Done caching stored playlists')


class DespotifyTranslator(object):
    @classmethod
    def to_mopidy_id(cls, spotify_uri):
        return spotify_uri_to_int(spotify_uri)

    @classmethod
    def to_mopidy_artist(cls, spotify_artist):
        return Artist(
            uri=spotify_artist.get_uri(),
            name=spotify_artist.name.decode(ENCODING)
        )

    @classmethod
    def to_mopidy_album(cls, spotify_album_name):
        return Album(name=spotify_album_name.decode(ENCODING))

    @classmethod
    def to_mopidy_track(cls, spotify_track):
        if spotify_track is None or not spotify_track.has_meta_data():
            return None
        if dt.MINYEAR <= int(spotify_track.year) <= dt.MAXYEAR:
            date = dt.date(spotify_track.year, 1, 1)
        else:
            date = None
        return Track(
            uri=spotify_track.get_uri(),
            name=spotify_track.title.decode(ENCODING),
            artists=[cls.to_mopidy_artist(a) for a in spotify_track.artists],
            album=cls.to_mopidy_album(spotify_track.album),
            track_no=spotify_track.tracknumber,
            date=date,
            length=spotify_track.length,
            bitrate=320,
            id=cls.to_mopidy_id(spotify_track.get_uri()),
        )

    @classmethod
    def to_mopidy_playlist(cls, spotify_playlist):
        return Playlist(
            uri=spotify_playlist.get_uri(),
            name=spotify_playlist.name.decode(ENCODING),
            tracks=filter(None,
                [cls.to_mopidy_track(t) for t in spotify_playlist.tracks]),
        )


class DespotifySessionManager(spytify.Spytify):
    DESPOTIFY_NEW_TRACK = 1
    DESPOTIFY_TIME_TELL = 2
    DESPOTIFY_END_OF_PLAYLIST = 3
    DESPOTIFY_TRACK_PLAY_ERROR = 4

    def __init__(self, *args, **kwargs):
        kwargs['callback'] = self.callback
        self.core_queue = kwargs.pop('core_queue')
        super(DespotifySessionManager, self).__init__(*args, **kwargs)

    def callback(self, signal, data):
        if signal == self.DESPOTIFY_END_OF_PLAYLIST:
            logger.debug('Despotify signalled end of playlist')
            self.core_queue.put({'command': 'end_of_track'})
        elif signal == self.DESPOTIFY_TRACK_PLAY_ERROR:
            logger.error('Despotify signalled track play error')



import gobject
gobject.threads_init()
# FIXME make sure we don't get hit by
# http://jameswestby.net/weblog/tech/14-caution-python-multiprocessing-and-glib-dont-mix.html

import pygst
pygst.require('0.10')

import gst
import logging
import os
import glob
import shutil
import threading

from mopidy.backends import *
from mopidy.models import Playlist, Track, Album
from mopidy import settings
from mopidy.utils import parse_m3u, parse_mpd_tag_cache

logger = logging.getLogger(u'backends.gstreamer')

class GStreamerMessages(threading.Thread):
    def run(self):
        gobject.MainLoop().run()

message_thread = GStreamerMessages()
message_thread.daemon = True
message_thread.start()

class GStreamerBackend(BaseBackend):
    """
    A backend for playing music from a local music archive.

    Uses the `GStreamer <http://gstreamer.freedesktop.org/>`_ library.

    **Issues:** http://github.com/jodal/mopidy/issues/labels/backend-gstreamer
    """

    def __init__(self, *args, **kwargs):
        super(GStreamerBackend, self).__init__(*args, **kwargs)

        self.library = GStreamerLibraryController(self)
        self.stored_playlists = GStreamerStoredPlaylistsController(self)
        self.current_playlist = BaseCurrentPlaylistController(self)
        self.playback = GStreamerPlaybackController(self)
        self.uri_handlers = [u'file://']


class GStreamerPlaybackController(BasePlaybackController):
    def __init__(self, backend):
        super(GStreamerPlaybackController, self).__init__(backend)

        self._bin = gst.element_factory_make("playbin", "player")
        self._bus = self._bin.get_bus()
        sink = gst.element_factory_make("fakesink", "fakesink")

        # FIXME cleanup fakesink?

        self._bin.set_property("video-sink", sink)
        self._bus.add_signal_watch()
        self._bus_id = self._bus.connect('message', self._message)

        self.stop()

    def use_fake_sink(self):
        """For testing. To avoid audio output during testing, and the need for
        a sound card and a fully working gstreamer installation."""
        sink = gst.element_factory_make("fakesink", "fakesink")
        self._bin.set_property("audio-sink", sink)

    def _set_state(self, state):
        self._bin.set_state(state)

        result, new, old = self._bin.get_state()

        return new == state

    def _message(self, bus, message):
        if message.type == gst.MESSAGE_EOS:
            self.end_of_track_callback()
        elif message.type == gst.MESSAGE_ERROR:
            self._bin.set_state(gst.STATE_NULL)
            error, debug = message.parse_error()
            logger.error('%s %s', error, debug)

    def _play(self, track):
        self._bin.set_state(gst.STATE_READY)
        self._bin.set_property('uri', track.uri)
        return self._set_state(gst.STATE_PLAYING)

    def _stop(self):
        return self._set_state(gst.STATE_READY)

    def _pause(self):
        return self._set_state(gst.STATE_PAUSED)

    def _resume(self):
        return self._set_state(gst.STATE_PLAYING)

    def _seek(self, time_position):
        self._bin.seek_simple(gst.Format(gst.FORMAT_TIME),
            gst.SEEK_FLAG_FLUSH, time_position * gst.MSECOND)
        self._set_state(gst.STATE_PLAYING)

    @property
    def time_position(self):
        try:
            return self._bin.query_position(gst.FORMAT_TIME)[0] // gst.MSECOND
        except gst.QueryError, e:
            logger.error('time_position failed: %s', e)
            return 0

    def destroy(self):
        playbin, self._bin = self._bin, None
        bus, self._bus = self._bus, None

        bus.disconnect(self._bus_id)
        bus.remove_signal_watch()
        playbin.get_state()
        playbin.set_state(gst.STATE_NULL)
        bus.set_flushing(True)

        del bus
        del playbin


class GStreamerStoredPlaylistsController(BaseStoredPlaylistsController):
    def __init__(self, *args):
        super(GStreamerStoredPlaylistsController, self).__init__(*args)
        self._folder = os.path.expanduser(settings.LOCAL_PLAYLIST_FOLDER)
        self.refresh()

    def refresh(self):
        playlists = []

        logger.info('Loading playlists from %s', self._folder)

        for m3u in glob.glob(os.path.join(self._folder, '*.m3u')):
            name = os.path.basename(m3u)[:len('.m3u')]
            tracks = []
            for uri in parse_m3u(m3u):
                try:
                    tracks.append(self.backend.library.lookup(uri))
                except LookupError, e:
                    logger.error('Playlist item could not be added: %s', e)
            playlist = Playlist(tracks=tracks, name=name)

            # FIXME playlist name needs better handling
            # FIXME tracks should come from lib. lookup

            playlists.append(playlist)

        self.playlists = playlists

    def create(self, name):
        playlist = Playlist(name=name)
        self.save(playlist)
        return playlist

    def delete(self, playlist):
        if playlist not in self._playlists:
            return

        self._playlists.remove(playlist)
        filename = os.path.join(self._folder, playlist.name + '.m3u')

        if os.path.exists(filename):
            os.remove(filename)

    def rename(self, playlist, name):
        if playlist not in self._playlists:
            return

        src = os.path.join(self._folder, playlist.name + '.m3u')
        dst = os.path.join(self._folder, name + '.m3u')

        renamed = playlist.with_(name=name)
        index = self._playlists.index(playlist)
        self._playlists[index] = renamed

        shutil.move(src, dst)

    def save(self, playlist):
        file_path = os.path.join(self._folder, playlist.name + '.m3u')

        # FIXME this should be a save_m3u function, not inside save
        with open(file_path, 'w') as file_handle:
            for track in playlist.tracks:
                if track.uri.startswith('file://'):
                    file_handle.write(track.uri[len('file://'):] + '\n')
                else:
                    file_handle.write(track.uri + '\n')

        self._playlists.append(playlist)


class GStreamerLibraryController(BaseLibraryController):
    def __init__(self, backend):
        super(GStreamerLibraryController, self).__init__(backend)
        self._uri_mapping = {}
        self.refresh()

    def refresh(self, uri=None):
        tracks = parse_mpd_tag_cache(settings.LOCAL_TAG_CACHE,
            settings.LOCAL_MUSIC_FOLDER)

        logger.info('Loading songs in %s from %s',
            settings.LOCAL_MUSIC_FOLDER, settings.LOCAL_TAG_CACHE)

        for track in tracks:
            self._uri_mapping[track.uri] = track

    def lookup(self, uri):
        try:
            return self._uri_mapping[uri]
        except KeyError:
            raise LookupError('%s not found.' % uri)

    def find_exact(self, field, query):
        if not query:
            raise LookupError('Missing query')

        if field == 'track':
            filter_func = lambda t: t.name == query
        elif field == 'album':
            filter_func = lambda t: getattr(t, 'album', Album()).name == query
        elif field == 'artist':
            filter_func = lambda t: filter(lambda a: a.name == query, t.artists)
        else:
            raise LookupError('Invalid lookup field: %s' % field)

        tracks = filter(filter_func, self._uri_mapping.values())
        return Playlist(tracks=tracks)

    def search(self, field, query):
        if not query:
            raise LookupError('Missing query')

        q = query.strip().lower()
        library_tracks = self._uri_mapping.values()

        # FIXME this is bound to be slow for large libraries
        track_filter  = lambda t: q in t.name.lower()
        album_filter = lambda t: q in getattr(t, 'album', Album()).name.lower()
        artist_filter = lambda t: filter(lambda a: q in a.name.lower(),
            t.artists)
        uri_filter = lambda t: q in t.uri.lower()
        any_filter = lambda t: track_filter(t) or album_filter(t) or \
            artist_filter(t) or uri_filter(t)

        if field == 'track':
            tracks = filter(track_filter, library_tracks)
        elif field == 'album':
            tracks = filter(album_filter, library_tracks)
        elif field == 'artist':
            tracks = filter(artist_filter, library_tracks)
        elif field == 'uri':
            tracks = filter(uri_filter, library_tracks)
        elif field == 'any':
            tracks = filter(any_filter, library_tracks)
        else:
            raise LookupError('Invalid lookup field: %s' % field)

        return Playlist(tracks=tracks)



from copy import copy
import logging
import random
import time

from mopidy import settings
from mopidy.models import Playlist
from mopidy.mpd import serializer
from mopidy.utils import get_class

logger = logging.getLogger('mopidy.backends.base')

__all__ = ['BaseBackend', 'BasePlaybackController',
    'BaseCurrentPlaylistController', 'BaseStoredPlaylistsController',
    'BaseLibraryController']

class BaseBackend(object):
    """
    :param core_queue: a queue for sending messages to
        :class:`mopidy.process.CoreProcess`
    :type core_queue: :class:`multiprocessing.Queue`
    :param mixer: either a mixer instance, or :class:`None` to use the mixer
        defined in settings
    :type mixer: :class:`mopidy.mixers.BaseMixer` or :class:`None`
    """

    def __init__(self, core_queue=None, mixer=None):
        self.core_queue = core_queue
        if mixer is not None:
            self.mixer = mixer
        else:
            self.mixer = get_class(settings.MIXER)()

    #: A :class:`multiprocessing.Queue` which can be used by e.g. library
    #: callbacks executing in other threads to send messages to the core
    #: thread, so that action may be taken in the correct thread.
    core_queue = None

    #: The current playlist controller. An instance of
    #: :class:`BaseCurrentPlaylistController`.
    current_playlist = None

    #: The library controller. An instance of :class:`BaseLibraryController`.
    library = None

    #: The sound mixer. An instance of :class:`mopidy.mixers.BaseMixer`.
    mixer = None

    #: The playback controller. An instance of :class:`BasePlaybackController`.
    playback = None

    #: The stored playlists controller. An instance of
    #: :class:`BaseStoredPlaylistsController`.
    stored_playlists = None

    #: List of URI prefixes this backend can handle.
    uri_handlers = []

    def destroy(self):
        """
        Call destroy on all sub-components in backend so that they can cleanup
        after themselves.
        """

        if self.current_playlist:
            self.current_playlist.destroy()

        if self.library:
            self.library.destroy()

        if self.mixer:
            self.mixer.destroy()

        if self.playback:
            self.playback.destroy()

        if self.stored_playlists:
            self.stored_playlists.destroy()

class BaseCurrentPlaylistController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`BaseBackend`
    """

    #: The current playlist version. Integer which is increased every time the
    #: current playlist is changed. Is not reset before Mopidy is restarted.
    version = 0

    def __init__(self, backend):
        self.backend = backend
        self._cp_tracks = []

    def destroy(self):
        """Cleanup after component."""
        pass

    @property
    def cp_tracks(self):
        """
        List of two-tuples of (CPID integer, :class:`mopidy.models.Track`).

        Read-only.
        """
        return [copy(ct) for ct in self._cp_tracks]

    @property
    def tracks(self):
        """
        List of :class:`mopidy.models.Track` in the current playlist.

        Read-only.
        """
        return [ct[1] for ct in self._cp_tracks]

    def add(self, track, at_position=None):
        """
        Add the track to the end of, or at the given position in the current
        playlist.

        :param track: track to add
        :type track: :class:`mopidy.models.Track`
        :param at_position: position in current playlist to add track
        :type at_position: int or :class:`None`
        :rtype: two-tuple of (CPID integer, :class:`mopidy.models.Track`) that
            was added to the current playlist playlist
        """
        assert at_position <= len(self._cp_tracks), \
            u'at_position can not be greater than playlist length'
        cp_track = (self.version, track)
        if at_position is not None:
            self._cp_tracks.insert(at_position, cp_track)
        else:
            self._cp_tracks.append(cp_track)
        self.version += 1
        return cp_track

    def clear(self):
        """Clear the current playlist."""
        self.backend.playback.stop()
        self.backend.playback.current_cp_track = None
        self._cp_tracks = []
        self.version += 1

    def get(self, **criteria):
        """
        Get track by given criterias from current playlist.

        Raises :exc:`LookupError` if a unique match is not found.

        Examples::

            get(cpid=7)             # Returns track with CPID 7
                                    # (current playlist ID)
            get(id=1)               # Returns track with ID 1
            get(uri='xyz')          # Returns track with URI 'xyz'
            get(id=1, uri='xyz')    # Returns track with ID 1 and URI 'xyz'

        :param criteria: on or more criteria to match by
        :type criteria: dict
        :rtype: two-tuple (CPID integer, :class:`mopidy.models.Track`)
        """
        matches = self._cp_tracks
        for (key, value) in criteria.iteritems():
            if key == 'cpid':
                matches = filter(lambda ct: ct[0] == value, matches)
            else:
                matches = filter(lambda ct: getattr(ct[1], key) == value,
                    matches)
        if len(matches) == 1:
            return matches[0]
        criteria_string = ', '.join(
            ['%s=%s' % (k, v) for (k, v) in criteria.iteritems()])
        if len(matches) == 0:
            raise LookupError(u'"%s" match no tracks' % criteria_string)
        else:
            raise LookupError(u'"%s" match multiple tracks' % criteria_string)

    def load(self, tracks):
        """
        Replace the tracks in the current playlist with the given tracks.

        :param tracks: tracks to load
        :type tracks: list of :class:`mopidy.models.Track`
        """
        self._cp_tracks = []
        self.version += 1
        for track in tracks:
            self.add(track)
        self.backend.playback.new_playlist_loaded_callback()

    def move(self, start, end, to_position):
        """
        Move the tracks in the slice ``[start:end]`` to ``to_position``.

        :param start: position of first track to move
        :type start: int
        :param end: position after last track to move
        :type end: int
        :param to_position: new position for the tracks
        :type to_position: int
        """
        if start == end:
            end += 1

        cp_tracks = self._cp_tracks

        assert start < end, 'start must be smaller than end'
        assert start >= 0, 'start must be at least zero'
        assert end <= len(cp_tracks), \
            'end can not be larger than playlist length'
        assert to_position >= 0, 'to_position must be at least zero'
        assert to_position <= len(cp_tracks), \
            'to_position can not be larger than playlist length'

        new_cp_tracks = cp_tracks[:start] + cp_tracks[end:]
        for cp_track in cp_tracks[start:end]:
            new_cp_tracks.insert(to_position, cp_track)
            to_position += 1
        self._cp_tracks = new_cp_tracks
        self.version += 1

    def remove(self, **criteria):
        """
        Remove the track from the current playlist.

        Uses :meth:`get()` to lookup the track to remove.

        :param criteria: on or more criteria to match by
        :type criteria: dict
        :type track: :class:`mopidy.models.Track`
        """
        cp_track = self.get(**criteria)
        position = self._cp_tracks.index(cp_track)
        del self._cp_tracks[position]
        self.version += 1

    def shuffle(self, start=None, end=None):
        """
        Shuffles the entire playlist. If ``start`` and ``end`` is given only
        shuffles the slice ``[start:end]``.

        :param start: position of first track to shuffle
        :type start: int or :class:`None`
        :param end: position after last track to shuffle
        :type end: int or :class:`None`
        """
        cp_tracks = self._cp_tracks

        if start is not None and end is not None:
            assert start < end, 'start must be smaller than end'

        if start is not None:
            assert start >= 0, 'start must be at least zero'

        if end is not None:
            assert end <= len(cp_tracks), 'end can not be larger than ' + \
                'playlist length'

        before = cp_tracks[:start or 0]
        shuffled = cp_tracks[start:end]
        after = cp_tracks[end or len(cp_tracks):]
        random.shuffle(shuffled)
        self._cp_tracks = before + shuffled + after
        self.version += 1

    def mpd_format(self, *args, **kwargs):
        """Not a part of the generic backend API."""
        kwargs['cpids'] = [ct[0] for ct in self._cp_tracks]
        return serializer.tracks_to_mpd_format(self.tracks, *args, **kwargs)


class BaseLibraryController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`BaseBackend`
    """

    def __init__(self, backend):
        self.backend = backend

    def destroy(self):
        """Cleanup after component."""
        pass

    def find_exact(self, field, query):
        """
        Find tracks in the library where ``field`` matches ``query`` exactly.

        :param field: 'track', 'artist', or 'album'
        :type field: string
        :param query: the search query
        :type query: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError

    def lookup(self, uri):
        """
        Lookup track with given URI. Returns :class:`None` if not found.

        :param uri: track URI
        :type uri: string
        :rtype: :class:`mopidy.models.Track` or :class:`None`
        """
        raise NotImplementedError

    def refresh(self, uri=None):
        """
        Refresh library. Limit to URI and below if an URI is given.

        :param uri: directory or track URI
        :type uri: string
        """
        raise NotImplementedError

    def search(self, field, query):
        """
        Search the library for tracks where ``field`` contains ``query``.

        :param field: 'track', 'artist', 'album', 'uri', and 'any'
        :type field: string
        :param query: the search query
        :type query: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError


class BasePlaybackController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`BaseBackend`
    """

    #: Constant representing the paused state.
    PAUSED = u'paused'

    #: Constant representing the playing state.
    PLAYING = u'playing'

    #: Constant representing the stopped state.
    STOPPED = u'stopped'

    #: :class:`True`
    #:     Tracks are removed from the playlist when they have been played.
    #: :class:`False`
    #:     Tracks are not removed from the playlist.
    consume = False

    #: The currently playing or selected track
    #:
    #: A two-tuple of (CPID integer, :class:`mopidy.models.Track`) or
    #: :class:`None`.
    current_cp_track = None

    #: :class:`True`
    #:     Tracks are selected at random from the playlist.
    #: :class:`False`
    #:     Tracks are played in the order of the playlist.
    random = False

    #: :class:`True`
    #:     The current playlist is played repeatedly. To repeat a single track,
    #:     select both :attr:`repeat` and :attr:`single`.
    #: :class:`False`
    #:     The current playlist is played once.
    repeat = False

    #: :class:`True`
    #:     Playback is stopped after current song, unless in repeat mode.
    #: :class:`False`
    #:     Playback continues after current song.
    single = False

    def __init__(self, backend):
        self.backend = backend
        self._state = self.STOPPED
        self._shuffled = []
        self._first_shuffle = True
        self._play_time_accumulated = 0
        self._play_time_started = None

    def destroy(self):
        """Cleanup after component."""
        pass

    @property
    def current_cpid(self):
        """
        The CPID (current playlist ID) of :attr:`current_track`.

        Read-only. Extracted from :attr:`current_cp_track` for convenience.
        """
        if self.current_cp_track is None:
            return None
        return self.current_cp_track[0]

    @property
    def current_track(self):
        """
        The currently playing or selected :class:`mopidy.models.Track`.

        Read-only. Extracted from :attr:`current_cp_track` for convenience.
        """
        if self.current_cp_track is None:
            return None
        return self.current_cp_track[1]

    @property
    def current_playlist_position(self):
        """The position of the current track in the current playlist."""
        if self.current_cp_track is None:
            return None
        try:
            return self.backend.current_playlist.cp_tracks.index(
                self.current_cp_track)
        except ValueError:
            return None

    @property
    def next_track(self):
        """
        The next track in the playlist.

        A :class:`mopidy.models.Track` extracted from :attr:`next_cp_track` for
        convenience.
        """
        next_cp_track = self.next_cp_track
        if next_cp_track is None:
            return None
        return next_cp_track[1]

    @property
    def next_cp_track(self):
        """
        The next track in the playlist.

        A two-tuple of (CPID integer, :class:`mopidy.models.Track`).

        For normal playback this is the next track in the playlist. If repeat
        is enabled the next track can loop around the playlist. When random is
        enabled this should be a random track, all tracks should be played once
        before the list repeats.
        """
        cp_tracks = self.backend.current_playlist.cp_tracks

        if not cp_tracks:
            return None

        if self.random and not self._shuffled:
            if self.repeat or self._first_shuffle:
                logger.debug('Shuffling tracks')
                self._shuffled = cp_tracks
                random.shuffle(self._shuffled)
                self._first_shuffle = False

        if self._shuffled:
            return self._shuffled[0]

        if self.current_cp_track is None:
            return cp_tracks[0]

        if self.repeat:
            return cp_tracks[
                (self.current_playlist_position + 1) % len(cp_tracks)]

        try:
            return cp_tracks[self.current_playlist_position + 1]
        except IndexError:
            return None

    @property
    def previous_track(self):
        """
        The previous track in the playlist.

        A :class:`mopidy.models.Track` extracted from :attr:`previous_cp_track`
        for convenience.
        """
        previous_cp_track = self.previous_cp_track
        if previous_cp_track is None:
            return None
        return previous_cp_track[1]

    @property
    def previous_cp_track(self):
        """
        The previous track in the playlist.

        A two-tuple of (CPID integer, :class:`mopidy.models.Track`).

        For normal playback this is the previous track in the playlist. If
        random and/or consume is enabled it should return the current track
        instead.
        """
        if self.repeat or self.consume or self.random:
            return self.current_cp_track

        if self.current_cp_track is None or self.current_playlist_position == 0:
            return None

        return self.backend.current_playlist.cp_tracks[
            self.current_playlist_position - 1]

    @property
    def state(self):
        """
        The playback state. Must be :attr:`PLAYING`, :attr:`PAUSED`, or
        :attr:`STOPPED`.

        Possible states and transitions:

        .. digraph:: state_transitions

            "STOPPED" -> "PLAYING" [ label="play" ]
            "PLAYING" -> "STOPPED" [ label="stop" ]
            "PLAYING" -> "PAUSED" [ label="pause" ]
            "PLAYING" -> "PLAYING" [ label="play" ]
            "PAUSED" -> "PLAYING" [ label="resume" ]
            "PAUSED" -> "STOPPED" [ label="stop" ]
        """
        return self._state

    @state.setter
    def state(self, new_state):
        (old_state, self._state) = (self.state, new_state)
        logger.debug(u'Changing state: %s -> %s', old_state, new_state)
        # FIXME _play_time stuff assumes backend does not have a better way of
        # handeling this stuff :/
        if (old_state in (self.PLAYING, self.STOPPED)
                and new_state == self.PLAYING):
            self._play_time_start()
        elif old_state == self.PLAYING and new_state == self.PAUSED:
            self._play_time_pause()
        elif old_state == self.PAUSED and new_state == self.PLAYING:
            self._play_time_resume()

    @property
    def time_position(self):
        """Time position in milliseconds."""
        if self.state == self.PLAYING:
            time_since_started = (self._current_wall_time -
                self._play_time_started)
            return self._play_time_accumulated + time_since_started
        elif self.state == self.PAUSED:
            return self._play_time_accumulated
        elif self.state == self.STOPPED:
            return 0

    def _play_time_start(self):
        self._play_time_accumulated = 0
        self._play_time_started = self._current_wall_time

    def _play_time_pause(self):
        time_since_started = self._current_wall_time - self._play_time_started
        self._play_time_accumulated += time_since_started

    def _play_time_resume(self):
        self._play_time_started = self._current_wall_time

    @property
    def _current_wall_time(self):
        return int(time.time() * 1000)

    def end_of_track_callback(self):
        """
        Tell the playback controller that end of track is reached.

        Typically called by :class:`mopidy.process.CoreProcess` after a message
        from a library thread is received.
        """
        if self.next_cp_track is not None:
            self.next()
        else:
            self.stop()
            self.current_cp_track = None

    def new_playlist_loaded_callback(self):
        """
        Tell the playback controller that a new playlist has been loaded.

        Typically called by :class:`mopidy.process.CoreProcess` after a message
        from a library thread is received.
        """
        self.current_cp_track = None
        self._first_shuffle = True
        self._shuffled = []

        if self.state == self.PLAYING:
            if len(self.backend.current_playlist.tracks) > 0:
                self.play()
            else:
                self.stop()
        elif self.state == self.PAUSED:
            self.stop()

    def next(self):
        """Play the next track."""
        original_cp_track = self.current_cp_track

        if self.state == self.STOPPED:
            return
        elif self.next_cp_track is not None and self._next(self.next_track):
            self.current_cp_track = self.next_cp_track
            self.state = self.PLAYING
        elif self.next_cp_track is None:
            self.stop()
            self.current_cp_track = None

        # FIXME handle in play aswell?
        if self.consume:
            self.backend.current_playlist.remove(cpid=original_cp_track[0])

        if self.random and self.current_cp_track in self._shuffled:
            self._shuffled.remove(self.current_cp_track)

    def _next(self, track):
        return self._play(track)

    def pause(self):
        """Pause playback."""
        if self.state == self.PLAYING and self._pause():
            self.state = self.PAUSED

    def _pause(self):
        raise NotImplementedError

    def play(self, cp_track=None):
        """
        Play the given track or the currently active track.

        :param cp_track: track to play
        :type cp_track: two-tuple (CPID integer, :class:`mopidy.models.Track`)
            or :class:`None`
        """

        if cp_track is not None:
            assert cp_track in self.backend.current_playlist.cp_tracks
        elif not self.current_cp_track:
            cp_track = self.next_cp_track

        if self.state == self.PAUSED and cp_track is None:
            self.resume()
        elif cp_track is not None and self._play(cp_track[1]):
            self.current_cp_track = cp_track
            self.state = self.PLAYING

        # TODO Do something sensible when _play() returns False, like calling
        # next(). Adding this todo instead of just implementing it as I want a
        # test case first.

        if self.random and self.current_cp_track in self._shuffled:
            self._shuffled.remove(self.current_cp_track)

    def _play(self, track):
        raise NotImplementedError

    def previous(self):
        """Play the previous track."""
        if (self.previous_cp_track is not None
                and self.state != self.STOPPED
                and self._previous(self.previous_track)):
            self.current_cp_track = self.previous_cp_track
            self.state = self.PLAYING

    def _previous(self, track):
        return self._play(track)

    def resume(self):
        """If paused, resume playing the current track."""
        if self.state == self.PAUSED and self._resume():
            self.state = self.PLAYING

    def _resume(self):
        raise NotImplementedError

    def seek(self, time_position):
        """
        Seeks to time position given in milliseconds.

        :param time_position: time position in milliseconds
        :type time_position: int
        """
        if self.state == self.STOPPED:
            self.play()
        elif self.state == self.PAUSED:
            self.resume()

        if time_position < 0:
            time_position = 0
        elif self.current_track and time_position > self.current_track.length:
            self.next()
            return

        self._seek(time_position)

    def _seek(self, time_position):
        raise NotImplementedError

    def stop(self):
        """Stop playing."""
        if self.state != self.STOPPED and self._stop():
            self.state = self.STOPPED

    def _stop(self):
        raise NotImplementedError


class BaseStoredPlaylistsController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`BaseBackend`
    """

    def __init__(self, backend):
        self.backend = backend
        self._playlists = []

    def destroy(self):
        """Cleanup after component."""
        pass

    @property
    def playlists(self):
        """List of :class:`mopidy.models.Playlist`."""
        return copy(self._playlists)

    @playlists.setter
    def playlists(self, playlists):
        self._playlists = playlists

    def create(self, name):
        """
        Create a new playlist.

        :param name: name of the new playlist
        :type name: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError

    def delete(self, playlist):
        """
        Delete playlist.

        :param playlist: the playlist to delete
        :type playlist: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError

    def get(self, **criteria):
        """
        Get playlist by given criterias from the set of stored playlists.

        Raises :exc:`LookupError` if a unique match is not found.

        Examples::

            get(name='a')            # Returns track with name 'a'
            get(uri='xyz')           # Returns track with URI 'xyz'
            get(name='a', uri='xyz') # Returns track with name 'a' and URI 'xyz'

        :param criteria: on or more criteria to match by
        :type criteria: dict
        :rtype: :class:`mopidy.models.Playlist`
        """
        matches = self._playlists
        for (key, value) in criteria.iteritems():
            matches = filter(lambda p: getattr(p, key) == value, matches)
        if len(matches) == 1:
            return matches[0]
        criteria_string = ', '.join(
            ['%s=%s' % (k, v) for (k, v) in criteria.iteritems()])
        if len(matches) == 0:
            raise LookupError('"%s" match no playlists' % criteria_string)
        else:
            raise LookupError('"%s" match multiple playlists' % criteria_string)

    def lookup(self, uri):
        """
        Lookup playlist with given URI in both the set of stored playlists and
        in any other playlist sources.

        :param uri: playlist URI
        :type uri: string
        :rtype: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError

    def refresh(self):
        """Refresh stored playlists."""
        raise NotImplementedError

    def rename(self, playlist, new_name):
        """
        Rename playlist.

        :param playlist: the playlist
        :type playlist: :class:`mopidy.models.Playlist`
        :param new_name: the new name
        :type new_name: string
        """
        raise NotImplementedError

    def save(self, playlist):
        """
        Save the playlist to the set of stored playlists.

        :param playlist: the playlist
        :type playlist: :class:`mopidy.models.Playlist`
        """
        raise NotImplementedError

    def search(self, query):
        """
        Search for playlists whose name contains ``query``.

        :param query: query to search for
        :type query: string
        :rtype: list of :class:`mopidy.models.Playlist`
        """
        return filter(lambda p: query in p.name, self._playlists)



from subprocess import Popen, PIPE
import time

from mopidy.mixers import BaseMixer

class OsaMixer(BaseMixer):
    """Mixer which uses ``osascript`` on OS X to control volume."""

    CACHE_TTL = 30

    _cache = None
    _last_update = None

    def _valid_cache(self):
        return (self._cache is not None
            and self._last_update is not None
            and (int(time.time() - self._last_update) < self.CACHE_TTL))

    def _get_volume(self):
        if not self._valid_cache():
            try:
                self._cache = int(Popen(
                    ['osascript', '-e',
                        'output volume of (get volume settings)'],
                    stdout=PIPE).communicate()[0])
            except ValueError:
                self._cache = None
            self._last_update = int(time.time())
        return self._cache

    def _set_volume(self, volume):
        Popen(['osascript', '-e', 'set volume output volume %d' % volume])
        self._cache = volume
        self._last_update = int(time.time())



import alsaaudio
import logging

from mopidy import settings
from mopidy.mixers import BaseMixer

logger = logging.getLogger('mopidy.mixers.alsa')

class AlsaMixer(BaseMixer):
    """
    Mixer which uses the Advanced Linux Sound Architecture (ALSA) to control
    volume.
    """

    def __init__(self, *args, **kwargs):
        super(AlsaMixer, self).__init__(*args, **kwargs)
        self._mixer = alsaaudio.Mixer(self._get_mixer_control())
        assert self._mixer is not None

    def _get_mixer_control(self):
        """Returns the first mixer control candidate that is known to ALSA"""
        candidates = self._get_mixer_control_candidates()
        for control in candidates:
            if control in alsaaudio.mixers():
                logger.info(u'Mixer control in use: %s', control)
                return control
            else:
                logger.debug(u'Mixer control not found, skipping: %s', control)
        logger.warning(u'No working mixer controls found. Tried: %s', candidates)

    def _get_mixer_control_candidates(self):
        """
        A mixer named 'Master' does not always exist, so we fall back to using
        'PCM'. If this does not work for you, you may set
        :attr:`mopidy.settings.MIXER_ALSA_CONTROL`.
        """
        if settings.MIXER_ALSA_CONTROL:
            return [settings.MIXER_ALSA_CONTROL]
        return [u'Master', u'PCM']

    def _get_volume(self):
        # FIXME does not seem to see external volume changes.
        return self._mixer.getvolume()[0]

    def _set_volume(self, volume):
        self._mixer.setvolume(volume)



from mopidy import settings as raw_settings

def get_version():
    return u'0.1.0a3'

def get_mpd_protocol_version():
    return u'0.16.0'

class MopidyException(Exception):
    def __init__(self, message):
        self.message = message

    @property
    def message(self):
        """Reimplement message field that was deprecated in Python 2.6"""
        return self._message

    @message.setter
    def message(self, message):
        self._message = message

class SettingsError(MopidyException):
    pass

class Settings(object):
    def __getattr__(self, attr):
        if attr.isupper() and not hasattr(raw_settings, attr):
            raise SettingsError(u'Setting "%s" is not set.' % attr)
        value = getattr(raw_settings, attr)
        if type(value) != bool and not value:
            raise SettingsError(u'Setting "%s" is empty.' % attr)
        return value

settings = Settings()



"""
This is our MPD protocol implementation.

This is partly based upon the `MPD protocol documentation
<http://www.musicpd.org/doc/protocol/>`_, which is a useful resource, but it is
rather incomplete with regards to data formats, both for requests and
responses. Thus, we have had to talk a great deal with the the original `MPD
server <http://mpd.wikia.com/>`_ using telnet to get the details we need to
implement our own MPD server which is compatible with the numerous existing
`MPD clients <http://mpd.wikia.com/wiki/Clients>`_.
"""

import datetime as dt
import logging
import re

from mopidy.mpd import (MpdAckError, MpdArgError, MpdUnknownCommand,
    MpdNoExistError, MpdNotImplemented)
from mopidy.utils import flatten

logger = logging.getLogger('mopidy.mpd.frontend')

_commands = set()
_request_handlers = {}

def handle_pattern(pattern):
    """
    Decorator for connecting command handlers to command patterns.

    If you use named groups in the pattern, the decorated method will get the
    groups as keyword arguments. If the group is optional, remember to give the
    argument a default value.

    For example, if the command is ``do that thing`` the ``what`` argument will
    be ``this thing``::

        @handle_pattern('^do (?P<what>.+)$')
        def do(what):
            ...

    :param pattern: regexp pattern for matching commands
    :type pattern: string
    """
    def decorator(func):
        match = re.search('([a-z_]+)', pattern)
        if match is not None:
            _commands.add(match.group())
        if pattern in _request_handlers:
            raise ValueError(u'Tried to redefine handler for %s with %s' % (
                pattern, func))
        _request_handlers[pattern] = func
        func.__doc__ = '        - **Pattern:** ``%s``\n\n%s' % (
            pattern, func.__doc__ or '')
        return func
    return decorator

class MpdFrontend(object):
    def __init__(self, backend=None):
        self.backend = backend
        self.command_list = False
        self.command_list_ok = False

    def handle_request(self, request, command_list_index=None):
        if self.command_list is not False and request != u'command_list_end':
            self.command_list.append(request)
            return None
        try:
            (handler, kwargs) = self.find_handler(request)
            result = handler(self, **kwargs)
        except MpdAckError as e:
            if command_list_index is not None:
                e.index = command_list_index
            return self.handle_response(e.get_mpd_ack(), add_ok=False)
        if request in (u'command_list_begin', u'command_list_ok_begin'):
            return None
        if command_list_index is not None:
            return self.handle_response(result, add_ok=False)
        return self.handle_response(result)

    def find_handler(self, request):
        for pattern in _request_handlers:
            matches = re.match(pattern, request)
            if matches is not None:
                return (_request_handlers[pattern], matches.groupdict())
        command = request.split(' ')[0]
        if command in _commands:
            raise MpdArgError(u'incorrect arguments', command=command)
        raise MpdUnknownCommand(command=command)

    def handle_response(self, result, add_ok=True):
        response = []
        if result is None:
            result = []
        elif not isinstance(result, list):
            result = [result]
        for line in flatten(result):
            if isinstance(line, dict):
                for (key, value) in line.items():
                    response.append(u'%s: %s' % (key, value))
            elif isinstance(line, tuple):
                (key, value) = line
                response.append(u'%s: %s' % (key, value))
            else:
                response.append(line)
        if add_ok and (not response or not response[-1].startswith(u'ACK')):
            response.append(u'OK')
        return response

    @handle_pattern(r'^disableoutput "(?P<outputid>\d+)"$')
    def _audio_output_disableoutput(self, outputid):
        """
        *musicpd.org, audio output section:*

            ``disableoutput``

            Turns an output off.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^enableoutput "(?P<outputid>\d+)"$')
    def _audio_output_enableoutput(self, outputid):
        """
        *musicpd.org, audio output section:*

            ``enableoutput``

            Turns an output on.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^outputs$')
    def _audio_output_outputs(self):
        """
        *musicpd.org, audio output section:*

            ``outputs``

            Shows information about all outputs.
        """
        return [
            ('outputid', 0),
            ('outputname', self.backend.__class__.__name__),
            ('outputenabled', 1),
        ]

    @handle_pattern(r'^command_list_begin$')
    def _command_list_begin(self):
        """
        *musicpd.org, command list section:*

            To facilitate faster adding of files etc. you can pass a list of
            commands all at once using a command list. The command list begins
            with ``command_list_begin`` or ``command_list_ok_begin`` and ends
            with ``command_list_end``.

            It does not execute any commands until the list has ended. The
            return value is whatever the return for a list of commands is. On
            success for all commands, ``OK`` is returned. If a command fails,
            no more commands are executed and the appropriate ``ACK`` error is
            returned. If ``command_list_ok_begin`` is used, ``list_OK`` is
            returned for each successful command executed in the command list.
        """
        self.command_list = []
        self.command_list_ok = False

    @handle_pattern(r'^command_list_end$')
    def _command_list_end(self):
        """See :meth:`_command_list_begin`."""
        if self.command_list is False:
            # Test for False exactly, and not e.g. empty list
            raise MpdUnknownCommand(command='command_list_end')
        (command_list, self.command_list) = (self.command_list, False)
        (command_list_ok, self.command_list_ok) = (self.command_list_ok, False)
        result = []
        for i, command in enumerate(command_list):
            response = self.handle_request(command, command_list_index=i)
            if response is not None:
                result.append(response)
            if response and response[-1].startswith(u'ACK'):
                return result
            if command_list_ok:
                response.append(u'list_OK')
        return result

    @handle_pattern(r'^command_list_ok_begin$')
    def _command_list_ok_begin(self):
        """See :meth:`_command_list_begin`."""
        self.command_list = []
        self.command_list_ok = True

    @handle_pattern(r'^close$')
    def _connection_close(self):
        """
        *musicpd.org, connection section:*

            ``close``

            Closes the connection to MPD.
        """
        # TODO Does not work after multiprocessing branch merge
        #self.session.do_close()

    @handle_pattern(r'^kill$')
    def _connection_kill(self):
        """
        *musicpd.org, connection section:*

            ``kill``

            Kills MPD.
        """
        # TODO Does not work after multiprocessing branch merge
        #self.session.do_kill()

    @handle_pattern(r'^password "(?P<password>[^"]+)"$')
    def _connection_password(self, password):
        """
        *musicpd.org, connection section:*

            ``password {PASSWORD}``

            This is used for authentication with the server. ``PASSWORD`` is
            simply the plaintext password.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^ping$')
    def _connection_ping(self):
        """
        *musicpd.org, connection section:*

            ``ping``

            Does nothing but return ``OK``.
        """
        pass

    @handle_pattern(r'^add "(?P<uri>[^"]*)"$')
    def _current_playlist_add(self, uri):
        """
        *musicpd.org, current playlist section:*

            ``add {URI}``

            Adds the file ``URI`` to the playlist (directories add recursively).
            ``URI`` can also be a single file.
        """
        track = self.backend.library.lookup(uri)
        if track is None:
            raise MpdNoExistError(
                u'directory or file not found', command=u'add')
        self.backend.current_playlist.add(track)

    @handle_pattern(r'^addid "(?P<uri>[^"]*)"( "(?P<songpos>\d+)")*$')
    def _current_playlist_addid(self, uri, songpos=None):
        """
        *musicpd.org, current playlist section:*

            ``addid {URI} [POSITION]``

            Adds a song to the playlist (non-recursive) and returns the song id.

            ``URI`` is always a single file or URL. For example::

                addid "foo.mp3"
                Id: 999
                OK
        """
        if songpos is not None:
            songpos = int(songpos)
        track = self.backend.library.lookup(uri)
        if track is None:
            raise MpdNoExistError(u'No such song', command=u'addid')
        if songpos and songpos > len(self.backend.current_playlist.tracks):
            raise MpdArgError(u'Bad song index', command=u'addid')
        cp_track = self.backend.current_playlist.add(track, at_position=songpos)
        return ('Id', cp_track[0])

    @handle_pattern(r'^delete "(?P<start>\d+):(?P<end>\d+)*"$')
    def _current_playlist_delete_range(self, start, end=None):
        """
        *musicpd.org, current playlist section:*

            ``delete [{POS} | {START:END}]``

            Deletes a song from the playlist.
        """
        start = int(start)
        if end is not None:
            end = int(end)
        else:
            end = len(self.backend.current_playlist.tracks)
        cp_tracks = self.backend.current_playlist.cp_tracks[start:end]
        if not cp_tracks:
            raise MpdArgError(u'Bad song index', command=u'delete')
        for (cpid, track) in cp_tracks:
            self.backend.current_playlist.remove(cpid=cpid)

    @handle_pattern(r'^delete "(?P<songpos>\d+)"$')
    def _current_playlist_delete_songpos(self, songpos):
        """See :meth:`_current_playlist_delete_range`"""
        try:
            songpos = int(songpos)
            (cpid, track) = self.backend.current_playlist.cp_tracks[songpos]
            self.backend.current_playlist.remove(cpid=cpid)
        except IndexError:
            raise MpdArgError(u'Bad song index', command=u'delete')

    @handle_pattern(r'^deleteid "(?P<cpid>\d+)"$')
    def _current_playlist_deleteid(self, cpid):
        """
        *musicpd.org, current playlist section:*

            ``deleteid {SONGID}``

            Deletes the song ``SONGID`` from the playlist
        """
        try:
            cpid = int(cpid)
            return self.backend.current_playlist.remove(cpid=cpid)
        except LookupError:
            raise MpdNoExistError(u'No such song', command=u'deleteid')

    @handle_pattern(r'^clear$')
    def _current_playlist_clear(self):
        """
        *musicpd.org, current playlist section:*

            ``clear``

            Clears the current playlist.
        """
        self.backend.current_playlist.clear()

    @handle_pattern(r'^move "(?P<start>\d+):(?P<end>\d+)*" "(?P<to>\d+)"$')
    def _current_playlist_move_range(self, start, to, end=None):
        """
        *musicpd.org, current playlist section:*

            ``move [{FROM} | {START:END}] {TO}``

            Moves the song at ``FROM`` or range of songs at ``START:END`` to
            ``TO`` in the playlist.
        """
        if end is None:
            end = len(self.backend.current_playlist.tracks)
        start = int(start)
        end = int(end)
        to = int(to)
        self.backend.current_playlist.move(start, end, to)

    @handle_pattern(r'^move "(?P<songpos>\d+)" "(?P<to>\d+)"$')
    def _current_playlist_move_songpos(self, songpos, to):
        """See :meth:`_current_playlist_move_range`."""
        songpos = int(songpos)
        to = int(to)
        self.backend.current_playlist.move(songpos, songpos + 1, to)

    @handle_pattern(r'^moveid "(?P<cpid>\d+)" "(?P<to>\d+)"$')
    def _current_playlist_moveid(self, cpid, to):
        """
        *musicpd.org, current playlist section:*

            ``moveid {FROM} {TO}``

            Moves the song with ``FROM`` (songid) to ``TO`` (playlist index) in
            the playlist. If ``TO`` is negative, it is relative to the current
            song in the playlist (if there is one).
        """
        cpid = int(cpid)
        to = int(to)
        cp_track = self.backend.current_playlist.get(cpid=cpid)
        position = self.backend.current_playlist.cp_tracks.index(cp_track)
        self.backend.current_playlist.move(position, position + 1, to)

    @handle_pattern(r'^playlist$')
    def _current_playlist_playlist(self):
        """
        *musicpd.org, current playlist section:*

            ``playlist``

            Displays the current playlist.

            .. note::

                Do not use this, instead use ``playlistinfo``.
        """
        return self._current_playlist_playlistinfo()

    @handle_pattern(r'^playlistfind (?P<tag>[^"]+) "(?P<needle>[^"]+)"$')
    @handle_pattern(r'^playlistfind "(?P<tag>[^"]+)" "(?P<needle>[^"]+)"$')
    def _current_playlist_playlistfind(self, tag, needle):
        """
        *musicpd.org, current playlist section:*

            ``playlistfind {TAG} {NEEDLE}``

            Finds songs in the current playlist with strict matching.

        *GMPC:*

        - does not add quotes around the tag.
        """
        if tag == 'filename':
            try:
                cp_track = self.backend.current_playlist.get(uri=needle)
                return cp_track[1].mpd_format()
            except LookupError:
                return None
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^playlistid( "(?P<cpid>\d+)")*$')
    def _current_playlist_playlistid(self, cpid=None):
        """
        *musicpd.org, current playlist section:*

            ``playlistid {SONGID}``

            Displays a list of songs in the playlist. ``SONGID`` is optional
            and specifies a single song to display info for.
        """
        if cpid is not None:
            try:
                cpid = int(cpid)
                cp_track = self.backend.current_playlist.get(cpid=cpid)
                position = self.backend.current_playlist.cp_tracks.index(
                    cp_track)
                return cp_track[1].mpd_format(position=position, cpid=cpid)
            except LookupError:
                raise MpdNoExistError(u'No such song', command=u'playlistid')
        else:
            return self.backend.current_playlist.mpd_format()

    @handle_pattern(r'^playlistinfo$')
    @handle_pattern(r'^playlistinfo "(?P<songpos>-?\d+)"$')
    @handle_pattern(r'^playlistinfo "(?P<start>\d+):(?P<end>\d+)*"$')
    def _current_playlist_playlistinfo(self, songpos=None,
            start=None, end=None):
        """
        *musicpd.org, current playlist section:*

            ``playlistinfo [[SONGPOS] | [START:END]]``

            Displays a list of all songs in the playlist, or if the optional
            argument is given, displays information only for the song
            ``SONGPOS`` or the range of songs ``START:END``.

        *ncmpc and mpc:*

        - uses negative indexes, like ``playlistinfo "-1"``, to request
          the entire playlist
        """
        if songpos == "-1":
            songpos = None

        if songpos is not None:
            songpos = int(songpos)
            start = songpos
            end = songpos + 1
            if start == -1:
                end = None
            return self.backend.current_playlist.mpd_format(start, end)
        else:
            if start is None:
                start = 0
            start = int(start)
            if not (0 <= start <= len(self.backend.current_playlist.tracks)):
                raise MpdArgError(u'Bad song index', command=u'playlistinfo')
            if end is not None:
                end = int(end)
                if end > len(self.backend.current_playlist.tracks):
                    end = None
            return self.backend.current_playlist.mpd_format(start, end)

    @handle_pattern(r'^playlistsearch "(?P<tag>[^"]+)" "(?P<needle>[^"]+)"$')
    @handle_pattern(r'^playlistsearch (?P<tag>\S+) "(?P<needle>[^"]+)"$')
    def _current_playlist_playlistsearch(self, tag, needle):
        """
        *musicpd.org, current playlist section:*

            ``playlistsearch {TAG} {NEEDLE}``

            Searches case-sensitively for partial matches in the current
            playlist.

        *GMPC:*

        - does not add quotes around the tag
        - uses ``filename`` and ``any`` as tags
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^plchanges "(?P<version>\d+)"$')
    def _current_playlist_plchanges(self, version):
        """
        *musicpd.org, current playlist section:*

            ``plchanges {VERSION}``

            Displays changed songs currently in the playlist since ``VERSION``.

            To detect songs that were deleted at the end of the playlist, use
            ``playlistlength`` returned by status command.
        """
        # XXX Naive implementation that returns all tracks as changed
        if int(version) < self.backend.current_playlist.version:
            return self.backend.current_playlist.mpd_format()

    @handle_pattern(r'^plchangesposid "(?P<version>\d+)"$')
    def _current_playlist_plchangesposid(self, version):
        """
        *musicpd.org, current playlist section:*

            ``plchangesposid {VERSION}``

            Displays changed songs currently in the playlist since ``VERSION``.
            This function only returns the position and the id of the changed
            song, not the complete metadata. This is more bandwidth efficient.

            To detect songs that were deleted at the end of the playlist, use
            ``playlistlength`` returned by status command.
        """
        # XXX Naive implementation that returns all tracks as changed
        if int(version) != self.backend.current_playlist.version:
            result = []
            for (position, (cpid, track)) in enumerate(
                    self.backend.current_playlist.cp_tracks):
                result.append((u'cpos', position))
                result.append((u'Id', cpid))
            return result

    @handle_pattern(r'^shuffle$')
    @handle_pattern(r'^shuffle "(?P<start>\d+):(?P<end>\d+)*"$')
    def _current_playlist_shuffle(self, start=None, end=None):
        """
        *musicpd.org, current playlist section:*

            ``shuffle [START:END]``

            Shuffles the current playlist. ``START:END`` is optional and
            specifies a range of songs.
        """
        if start is not None:
            start = int(start)
        if end is not None:
            end = int(end)
        self.backend.current_playlist.shuffle(start, end)

    @handle_pattern(r'^swap "(?P<songpos1>\d+)" "(?P<songpos2>\d+)"$')
    def _current_playlist_swap(self, songpos1, songpos2):
        """
        *musicpd.org, current playlist section:*

            ``swap {SONG1} {SONG2}``

            Swaps the positions of ``SONG1`` and ``SONG2``.
        """
        songpos1 = int(songpos1)
        songpos2 = int(songpos2)
        tracks = self.backend.current_playlist.tracks
        song1 = tracks[songpos1]
        song2 = tracks[songpos2]
        del tracks[songpos1]
        tracks.insert(songpos1, song2)
        del tracks[songpos2]
        tracks.insert(songpos2, song1)
        self.backend.current_playlist.load(tracks)

    @handle_pattern(r'^swapid "(?P<cpid1>\d+)" "(?P<cpid2>\d+)"$')
    def _current_playlist_swapid(self, cpid1, cpid2):
        """
        *musicpd.org, current playlist section:*

            ``swapid {SONG1} {SONG2}``

            Swaps the positions of ``SONG1`` and ``SONG2`` (both song ids).
        """
        cpid1 = int(cpid1)
        cpid2 = int(cpid2)
        cp_track1 = self.backend.current_playlist.get(cpid=cpid1)
        cp_track2 = self.backend.current_playlist.get(cpid=cpid2)
        position1 = self.backend.current_playlist.cp_tracks.index(cp_track1)
        position2 = self.backend.current_playlist.cp_tracks.index(cp_track2)
        self._current_playlist_swap(position1, position2)

    @handle_pattern(r'^$')
    def _empty(self):
        """The original MPD server returns ``OK`` on an empty request.``"""
        pass

    @handle_pattern(r'^count "(?P<tag>[^"]+)" "(?P<needle>[^"]*)"$')
    def _music_db_count(self, tag, needle):
        """
        *musicpd.org, music database section:*

            ``count {TAG} {NEEDLE}``

            Counts the number of songs and their total playtime in the db
            matching ``TAG`` exactly.
        """
        return [('songs', 0), ('playtime', 0)] # TODO

    @handle_pattern(r'^find (?P<field>([Aa]lbum|[Aa]rtist|[Tt]itle)) '
        r'"(?P<what>[^"]+)"$')
    @handle_pattern(r'^find "(?P<field>(album|artist|title))" '
        r'"(?P<what>[^"]+)"$')
    @handle_pattern(r'^find (?P<field>(album)) '
        r'"(?P<what>[^"]+)" artist "([^"]+)"$')
    def _music_db_find(self, field, what):
        """
        *musicpd.org, music database section:*

            ``find {TYPE} {WHAT}``

            Finds songs in the db that are exactly ``WHAT``. ``TYPE`` should be
            ``album``, ``artist``, or ``title``. ``WHAT`` is what to find.

        *GMPC:*

        - does not add quotes around the field argument.
        - also uses ``find album "[ALBUM]" artist "[ARTIST]"`` to list album
          tracks.

        *ncmpc:*

        - does not add quotes around the field argument.
        - capitalizes the type argument.
        """
        field = field.lower()
        if field == u'title':
            field = u'track'
        return self.backend.library.find_exact(field, what).mpd_format()

    @handle_pattern(r'^findadd "(?P<field>(album|artist|title))" '
        r'"(?P<what>[^"]+)"$')
    def _music_db_findadd(self, field, what):
        """
        *musicpd.org, music database section:*

            ``findadd {TYPE} {WHAT}``

            Finds songs in the db that are exactly ``WHAT`` and adds them to
            current playlist. ``TYPE`` can be any tag supported by MPD.
            ``WHAT`` is what to find.
        """
        result = self._music_db_find(field, what)
        # TODO Add result to current playlist
        #return result

    @handle_pattern(r'^list (?P<field>[Aa]rtist)$')
    @handle_pattern(r'^list "(?P<field>[Aa]rtist)"$')
    @handle_pattern(r'^list (?P<field>album( artist)?)( "(?P<artist>[^"]+)")*$')
    @handle_pattern(r'^list "(?P<field>album(" "artist)?)"( "(?P<artist>[^"]+)")*$')
    def _music_db_list(self, field, artist=None):
        """
        *musicpd.org, music database section:*

            ``list {TYPE} [ARTIST]``

            Lists all tags of the specified type. ``TYPE`` should be ``album``
            or ``artist``.

            ``ARTIST`` is an optional parameter when type is ``album``, this
            specifies to list albums by an artist.

        *GMPC:*

        - does not add quotes around the field argument.
        - asks for multiple fields, i.e.::

            list album artist "an artist name"

          returns the albums available for the asked artist::

            list album artist "Tiesto"
            Album: Radio Trance Vol 4-Promo-CD
            Album: Ur  A Tear in the Open CDR
            Album: Simple Trance 2004 Step One
            Album: In Concert 05-10-2003

        *ncmpc:*

        - does not add quotes around the field argument.
        - capitalizes the field argument.
        """
        field = field.lower()
        pass # TODO

    @handle_pattern(r'^listall "(?P<uri>[^"]+)"')
    def _music_db_listall(self, uri):
        """
        *musicpd.org, music database section:*

            ``listall [URI]``

            Lists all songs and directories in ``URI``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^listallinfo "(?P<uri>[^"]+)"')
    def _music_db_listallinfo(self, uri):
        """
        *musicpd.org, music database section:*

            ``listallinfo [URI]``

            Same as ``listall``, except it also returns metadata info in the
            same format as ``lsinfo``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^lsinfo$')
    @handle_pattern(r'^lsinfo "(?P<uri>[^"]*)"$')
    def _music_db_lsinfo(self, uri=None):
        """
        *musicpd.org, music database section:*

            ``lsinfo [URI]``

            Lists the contents of the directory ``URI``.

            When listing the root directory, this currently returns the list of
            stored playlists. This behavior is deprecated; use
            ``listplaylists`` instead.

        MPD returns the same result, including both playlists and the files and
        directories located at the root level, for both ``lsinfo``, ``lsinfo
        ""``, and ``lsinfo "/"``.
        """
        if uri is None or uri == u'/' or uri == u'':
            return self._stored_playlists_listplaylists()
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^rescan( "(?P<uri>[^"]+)")*$')
    def _music_db_rescan(self, uri=None):
        """
        *musicpd.org, music database section:*

            ``rescan [URI]``

            Same as ``update``, but also rescans unmodified files.
        """
        return self._music_db_update(uri, rescan_unmodified_files=True)

    @handle_pattern(r'^search '
        r'(?P<field>([Aa]lbum|[Aa]rtist|[Ff]ilename|[Tt]itle|[Aa]ny)) '
        r'"(?P<what>[^"]+)"$')
    @handle_pattern(r'^search "(?P<field>(album|artist|filename|title|any))" '
        r'"(?P<what>[^"]+)"$')
    def _music_db_search(self, field, what):
        """
        *musicpd.org, music database section:*

            ``search {TYPE} {WHAT}``

            Searches for any song that contains ``WHAT``. ``TYPE`` can be
            ``title``, ``artist``, ``album`` or ``filename``. Search is not
            case sensitive.

        *GMPC:*

        - does not add quotes around the field argument.
        - uses the undocumented field ``any``.
        - searches for multiple words like this::

            search any "foo" any "bar" any "baz"

        *ncmpc:*

        - does not add quotes around the field argument.
        - capitalizes the field argument.
        """
        # TODO Support GMPC multi-word search
        field = field.lower()
        if field == u'title':
            field = u'track'
        return self.backend.library.search(field, what).mpd_format()

    @handle_pattern(r'^update( "(?P<uri>[^"]+)")*$')
    def _music_db_update(self, uri=None, rescan_unmodified_files=False):
        """
        *musicpd.org, music database section:*

            ``update [URI]``

            Updates the music database: find new files, remove deleted files,
            update modified files.

            ``URI`` is a particular directory or song/file to update. If you do
            not specify it, everything is updated.

            Prints ``updating_db: JOBID`` where ``JOBID`` is a positive number
            identifying the update job. You can read the current job id in the
            ``status`` response.
        """
        return {'updating_db': 0} # TODO

    @handle_pattern(r'^consume "(?P<state>[01])"$')
    def _playback_consume(self, state):
        """
        *musicpd.org, playback section:*

            ``consume {STATE}``

            Sets consume state to ``STATE``, ``STATE`` should be 0 or
            1. When consume is activated, each song played is removed from
            playlist.
        """
        if int(state):
            self.backend.playback.consume = True
        else:
            self.backend.playback.consume = False

    @handle_pattern(r'^crossfade "(?P<seconds>\d+)"$')
    def _playback_crossfade(self, seconds):
        """
        *musicpd.org, playback section:*

            ``crossfade {SECONDS}``

            Sets crossfading between songs.
        """
        seconds = int(seconds)
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^next$')
    def _playback_next(self):
        """
        *musicpd.org, playback section:*

            ``next``

            Plays next song in the playlist.

        *MPD's behaviour when affected by repeat/random/single/consume:*

            Given a playlist of three tracks numbered 1, 2, 3, and a currently
            playing track ``c``. ``next_track`` is defined at the track that
            will be played upon calls to ``next``.

            Tests performed on MPD 0.15.4-1ubuntu3.

            ======  ======  ======  =======  =====  =====  =====  =====
                        Inputs                    next_track
            -------------------------------  -------------------  -----
            repeat  random  single  consume  c = 1  c = 2  c = 3  Notes
            ======  ======  ======  =======  =====  =====  =====  =====
            T       T       T       T        2      3      EOPL
            T       T       T       .        Rand   Rand   Rand   [1]
            T       T       .       T        Rand   Rand   Rand   [4]
            T       T       .       .        Rand   Rand   Rand   [4]
            T       .       T       T        2      3      EOPL
            T       .       T       .        2      3      1
            T       .       .       T        3      3      EOPL
            T       .       .       .        2      3      1
            .       T       T       T        Rand   Rand   Rand   [3]
            .       T       T       .        Rand   Rand   Rand   [3]
            .       T       .       T        Rand   Rand   Rand   [2]
            .       T       .       .        Rand   Rand   Rand   [2]
            .       .       T       T        2      3      EOPL
            .       .       T       .        2      3      EOPL
            .       .       .       T        2      3      EOPL
            .       .       .       .        2      3      EOPL
            ======  ======  ======  =======  =====  =====  =====  =====

            - When end of playlist (EOPL) is reached, the current track is
              unset.
            - [1] When *random* and *single* is combined, ``next`` selects
              a track randomly at each invocation, and not just the next track
              in an internal prerandomized playlist.
            - [2] When *random* is active, ``next`` will skip through
              all tracks in the playlist in random order, and finally EOPL is
              reached.
            - [3] *single* has no effect in combination with *random*
              alone, or *random* and *consume*.
            - [4] When *random* and *repeat* is active, EOPL is never
              reached, but the playlist is played again, in the same random
              order as the first time.

        """
        return self.backend.playback.next()

    @handle_pattern(r'^pause "(?P<state>[01])"$')
    def _playback_pause(self, state):
        """
        *musicpd.org, playback section:*

            ``pause {PAUSE}``

            Toggles pause/resumes playing, ``PAUSE`` is 0 or 1.
        """
        if int(state):
            self.backend.playback.pause()
        else:
            self.backend.playback.resume()

    @handle_pattern(r'^play$')
    def _playback_play(self):
        """
        The original MPD server resumes from the paused state on ``play``
        without arguments.
        """
        return self.backend.playback.play()

    @handle_pattern(r'^playid "(?P<cpid>\d+)"$')
    @handle_pattern(r'^playid "(?P<cpid>-1)"$')
    def _playback_playid(self, cpid):
        """
        *musicpd.org, playback section:*

            ``playid [SONGID]``

            Begins playing the playlist at song ``SONGID``.

        *GMPC:*

        - issues ``playid "-1"`` after playlist replacement to start playback
          at the first track.
        """
        cpid = int(cpid)
        try:
            if cpid == -1:
                cp_track = self.backend.current_playlist.cp_tracks[0]
            else:
                cp_track = self.backend.current_playlist.get(cpid=cpid)
            return self.backend.playback.play(cp_track)
        except LookupError:
            raise MpdNoExistError(u'No such song', command=u'playid')

    @handle_pattern(r'^play "(?P<songpos>\d+)"$')
    @handle_pattern(r'^play "(?P<songpos>-1)"$')
    def _playback_playpos(self, songpos):
        """
        *musicpd.org, playback section:*

            ``play [SONGPOS]``

            Begins playing the playlist at song number ``SONGPOS``.

        *MPoD:*

        - issues ``play "-1"`` after playlist replacement to start playback at
          the first track.
        """
        songpos = int(songpos)
        try:
            if songpos == -1:
                cp_track = self.backend.current_playlist.cp_tracks[0]
            else:
                cp_track = self.backend.current_playlist.cp_tracks[songpos]
            return self.backend.playback.play(cp_track)
        except IndexError:
            raise MpdArgError(u'Bad song index', command=u'play')

    @handle_pattern(r'^previous$')
    def _playback_previous(self):
        """
        *musicpd.org, playback section:*

            ``previous``

            Plays previous song in the playlist.

        *MPD's behaviour when affected by repeat/random/single/consume:*

            Given a playlist of three tracks numbered 1, 2, 3, and a currently
            playing track ``c``. ``previous_track`` is defined at the track
            that will be played upon ``previous`` calls.

            Tests performed on MPD 0.15.4-1ubuntu3.

            ======  ======  ======  =======  =====  =====  =====
                        Inputs                  previous_track
            -------------------------------  -------------------
            repeat  random  single  consume  c = 1  c = 2  c = 3
            ======  ======  ======  =======  =====  =====  =====
            T       T       T       T        Rand?  Rand?  Rand?
            T       T       T       .        3      1      2
            T       T       .       T        Rand?  Rand?  Rand?
            T       T       .       .        3      1      2
            T       .       T       T        3      1      2
            T       .       T       .        3      1      2
            T       .       .       T        3      1      2
            T       .       .       .        3      1      2
            .       T       T       T        c      c      c
            .       T       T       .        c      c      c
            .       T       .       T        c      c      c
            .       T       .       .        c      c      c
            .       .       T       T        1      1      2
            .       .       T       .        1      1      2
            .       .       .       T        1      1      2
            .       .       .       .        1      1      2
            ======  ======  ======  =======  =====  =====  =====

            - If :attr:`time_position` of the current track is 15s or more,
              ``previous`` should do a seek to time position 0.

        """
        return self.backend.playback.previous()

    @handle_pattern(r'^random "(?P<state>[01])"$')
    def _playback_random(self, state):
        """
        *musicpd.org, playback section:*

            ``random {STATE}``

            Sets random state to ``STATE``, ``STATE`` should be 0 or 1.
        """
        if int(state):
            self.backend.playback.random = True
        else:
            self.backend.playback.random = False

    @handle_pattern(r'^repeat "(?P<state>[01])"$')
    def _playback_repeat(self, state):
        """
        *musicpd.org, playback section:*

            ``repeat {STATE}``

            Sets repeat state to ``STATE``, ``STATE`` should be 0 or 1.
        """
        if int(state):
            self.backend.playback.repeat = True
        else:
            self.backend.playback.repeat = False

    @handle_pattern(r'^replay_gain_mode "(?P<mode>(off|track|album))"$')
    def _playback_replay_gain_mode(self, mode):
        """
        *musicpd.org, playback section:*

            ``replay_gain_mode {MODE}``

            Sets the replay gain mode. One of ``off``, ``track``, ``album``.

            Changing the mode during playback may take several seconds, because
            the new settings does not affect the buffered data.

            This command triggers the options idle event.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^replay_gain_status$')
    def _playback_replay_gain_status(self):
        """
        *musicpd.org, playback section:*

            ``replay_gain_status``

            Prints replay gain options. Currently, only the variable
            ``replay_gain_mode`` is returned.
        """
        return u'off' # TODO

    @handle_pattern(r'^seek "(?P<songpos>\d+)" "(?P<seconds>\d+)"$')
    def _playback_seek(self, songpos, seconds):
        """
        *musicpd.org, playback section:*

            ``seek {SONGPOS} {TIME}``

            Seeks to the position ``TIME`` (in seconds) of entry ``SONGPOS`` in
            the playlist.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^seekid "(?P<cpid>\d+)" "(?P<seconds>\d+)"$')
    def _playback_seekid(self, cpid, seconds):
        """
        *musicpd.org, playback section:*

            ``seekid {SONGID} {TIME}``

            Seeks to the position ``TIME`` (in seconds) of song ``SONGID``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^setvol "(?P<volume>[-+]*\d+)"$')
    def _playback_setvol(self, volume):
        """
        *musicpd.org, playback section:*

            ``setvol {VOL}``

            Sets volume to ``VOL``, the range of volume is 0-100.
        """
        volume = int(volume)
        if volume < 0:
            volume = 0
        if volume > 100:
            volume = 100
        self.backend.mixer.volume = volume

    @handle_pattern(r'^single "(?P<state>[01])"$')
    def _playback_single(self, state):
        """
        *musicpd.org, playback section:*

            ``single {STATE}``

            Sets single state to ``STATE``, ``STATE`` should be 0 or 1. When
            single is activated, playback is stopped after current song, or
            song is repeated if the ``repeat`` mode is enabled.
        """
        if int(state):
            self.backend.playback.single = True
        else:
            self.backend.playback.single = False

    @handle_pattern(r'^stop$')
    def _playback_stop(self):
        """
        *musicpd.org, playback section:*

            ``stop``

            Stops playing.
        """
        self.backend.playback.stop()

    @handle_pattern(r'^commands$')
    def _reflection_commands(self):
        """
        *musicpd.org, reflection section:*

            ``commands``

            Shows which commands the current user has access to.

        As permissions is not implemented, any user has access to all commands.
        """
        commands = sorted(list(_commands))

        # Not shown by MPD in its command list
        commands.remove('command_list_begin')
        commands.remove('command_list_ok_begin')
        commands.remove('command_list_end')
        commands.remove('idle')
        commands.remove('noidle')
        commands.remove('sticker')

        return [('command', c) for c in commands]

    @handle_pattern(r'^decoders$')
    def _reflection_decoders(self):
        """
        *musicpd.org, reflection section:*

            ``decoders``

            Print a list of decoder plugins, followed by their supported
            suffixes and MIME types. Example response::

                plugin: mad
                suffix: mp3
                suffix: mp2
                mime_type: audio/mpeg
                plugin: mpcdec
                suffix: mpc
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^notcommands$')
    def _reflection_notcommands(self):
        """
        *musicpd.org, reflection section:*

            ``notcommands``

            Shows which commands the current user does not have access to.

        As permissions is not implemented, any user has access to all commands.
        """
        pass

    @handle_pattern(r'^tagtypes$')
    def _reflection_tagtypes(self):
        """
        *musicpd.org, reflection section:*

            ``tagtypes``

            Shows a list of available song metadata.
        """
        pass # TODO

    @handle_pattern(r'^urlhandlers$')
    def _reflection_urlhandlers(self):
        """
        *musicpd.org, reflection section:*

            ``urlhandlers``

            Gets a list of available URL handlers.
        """
        return [(u'handler', uri) for uri in self.backend.uri_handlers]

    @handle_pattern(r'^clearerror$')
    def _status_clearerror(self):
        """
        *musicpd.org, status section:*

            ``clearerror``

            Clears the current error message in status (this is also
            accomplished by any command that starts playback).
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^currentsong$')
    def _status_currentsong(self):
        """
        *musicpd.org, status section:*

            ``currentsong``

            Displays the song info of the current song (same song that is
            identified in status).
        """
        if self.backend.playback.current_track is not None:
            return self.backend.playback.current_track.mpd_format(
                position=self.backend.playback.current_playlist_position,
                cpid=self.backend.playback.current_cpid)

    @handle_pattern(r'^idle$')
    @handle_pattern(r'^idle (?P<subsystems>.+)$')
    def _status_idle(self, subsystems=None):
        """
        *musicpd.org, status section:*

            ``idle [SUBSYSTEMS...]``

            Waits until there is a noteworthy change in one or more of MPD's
            subsystems. As soon as there is one, it lists all changed systems
            in a line in the format ``changed: SUBSYSTEM``, where ``SUBSYSTEM``
            is one of the following:

            - ``database``: the song database has been modified after update.
            - ``update``: a database update has started or finished. If the
              database was modified during the update, the database event is
              also emitted.
            - ``stored_playlist``: a stored playlist has been modified,
              renamed, created or deleted
            - ``playlist``: the current playlist has been modified
            - ``player``: the player has been started, stopped or seeked
            - ``mixer``: the volume has been changed
            - ``output``: an audio output has been enabled or disabled
            - ``options``: options like repeat, random, crossfade, replay gain

            While a client is waiting for idle results, the server disables
            timeouts, allowing a client to wait for events as long as MPD runs.
            The idle command can be canceled by sending the command ``noidle``
            (no other commands are allowed). MPD will then leave idle mode and
            print results immediately; might be empty at this time.

            If the optional ``SUBSYSTEMS`` argument is used, MPD will only send
            notifications when something changed in one of the specified
            subsystems.
        """
        pass # TODO

    @handle_pattern(r'^noidle$')
    def _status_noidle(self):
        """See :meth:`_status_idle`."""
        pass # TODO

    @handle_pattern(r'^stats$')
    def _status_stats(self):
        """
        *musicpd.org, status section:*

            ``stats``

            Displays statistics.

            - ``artists``: number of artists
            - ``songs``: number of albums
            - ``uptime``: daemon uptime in seconds
            - ``db_playtime``: sum of all song times in the db
            - ``db_update``: last db update in UNIX time
            - ``playtime``: time length of music played
        """
        return {
            'artists': 0, # TODO
            'albums': 0, # TODO
            'songs': 0, # TODO
            # TODO Does not work after multiprocessing branch merge
            'uptime': 0, # self.session.stats_uptime(),
            'db_playtime': 0, # TODO
            'db_update': 0, # TODO
            'playtime': 0, # TODO
        }

    @handle_pattern(r'^status$')
    def _status_status(self):
        """
        *musicpd.org, status section:*

            ``status``

            Reports the current status of the player and the volume level.

            - ``volume``: 0-100
            - ``repeat``: 0 or 1
            - ``single``: 0 or 1
            - ``consume``: 0 or 1
            - ``playlist``: 31-bit unsigned integer, the playlist version
              number
            - ``playlistlength``: integer, the length of the playlist
            - ``state``: play, stop, or pause
            - ``song``: playlist song number of the current song stopped on or
              playing
            - ``songid``: playlist songid of the current song stopped on or
              playing
            - ``nextsong``: playlist song number of the next song to be played
            - ``nextsongid``: playlist songid of the next song to be played
            - ``time``: total time elapsed (of current playing/paused song)
            - ``elapsed``: Total time elapsed within the current song, but with
              higher resolution.
            - ``bitrate``: instantaneous bitrate in kbps
            - ``xfade``: crossfade in seconds
            - ``audio``: sampleRate``:bits``:channels
            - ``updatings_db``: job id
            - ``error``: if there is an error, returns message here
        """
        result = [
            ('volume', self.__status_status_volume()),
            ('repeat', self.__status_status_repeat()),
            ('random', self.__status_status_random()),
            ('single', self.__status_status_single()),
            ('consume', self.__status_status_consume()),
            ('playlist', self.__status_status_playlist_version()),
            ('playlistlength', self.__status_status_playlist_length()),
            ('xfade', self.__status_status_xfade()),
            ('state', self.__status_status_state()),
        ]
        if self.backend.playback.current_track is not None:
            result.append(('song', self.__status_status_songpos()))
            result.append(('songid', self.__status_status_songid()))
        if self.backend.playback.state in (
                self.backend.playback.PLAYING, self.backend.playback.PAUSED):
            result.append(('time', self.__status_status_time()))
            result.append(('elapsed', self.__status_status_time_elapsed()))
            result.append(('bitrate', self.__status_status_bitrate()))
        return result

    def __status_status_bitrate(self):
        if self.backend.playback.current_track is not None:
            return self.backend.playback.current_track.bitrate

    def __status_status_consume(self):
        if self.backend.playback.consume:
            return 1
        else:
            return 0

    def __status_status_playlist_length(self):
        return len(self.backend.current_playlist.tracks)

    def __status_status_playlist_version(self):
        return self.backend.current_playlist.version

    def __status_status_random(self):
        return int(self.backend.playback.random)

    def __status_status_repeat(self):
        return int(self.backend.playback.repeat)

    def __status_status_single(self):
        return int(self.backend.playback.single)

    def __status_status_songid(self):
        if self.backend.playback.current_cpid is not None:
            return self.backend.playback.current_cpid
        else:
            return self.__status_status_songpos()

    def __status_status_songpos(self):
        return self.backend.playback.current_playlist_position

    def __status_status_state(self):
        if self.backend.playback.state == self.backend.playback.PLAYING:
            return u'play'
        elif self.backend.playback.state == self.backend.playback.STOPPED:
            return u'stop'
        elif self.backend.playback.state == self.backend.playback.PAUSED:
            return u'pause'

    def __status_status_time(self):
        return u'%s:%s' % (self.__status_status_time_elapsed() // 1000,
            self.__status_status_time_total() // 1000)

    def __status_status_time_elapsed(self):
        return self.backend.playback.time_position

    def __status_status_time_total(self):
        if self.backend.playback.current_track is None:
            return 0
        elif self.backend.playback.current_track.length is None:
            return 0
        else:
            return self.backend.playback.current_track.length

    def __status_status_volume(self):
        if self.backend.mixer.volume is not None:
            return self.backend.mixer.volume
        else:
            return 0

    def __status_status_xfade(self):
        return 0 # TODO

    @handle_pattern(r'^sticker delete "(?P<field>[^"]+)" '
        r'"(?P<uri>[^"]+)"( "(?P<name>[^"]+)")*$')
    def _sticker_delete(self, field, uri, name=None):
        """
        *musicpd.org, sticker section:*

            ``sticker delete {TYPE} {URI} [NAME]``

            Deletes a sticker value from the specified object. If you do not
            specify a sticker name, all sticker values are deleted.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^sticker find "(?P<field>[^"]+)" "(?P<uri>[^"]+)" '
        r'"(?P<name>[^"]+)"$')
    def _sticker_find(self, field, uri, name):
        """
        *musicpd.org, sticker section:*

            ``sticker find {TYPE} {URI} {NAME}``

            Searches the sticker database for stickers with the specified name,
            below the specified directory (``URI``). For each matching song, it
            prints the ``URI`` and that one sticker's value.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^sticker get "(?P<field>[^"]+)" "(?P<uri>[^"]+)" '
        r'"(?P<name>[^"]+)"$')
    def _sticker_get(self, field, uri, name):
        """
        *musicpd.org, sticker section:*

            ``sticker get {TYPE} {URI} {NAME}``

            Reads a sticker value for the specified object.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^sticker list "(?P<field>[^"]+)" "(?P<uri>[^"]+)"$')
    def _sticker_list(self, field, uri):
        """
        *musicpd.org, sticker section:*

            ``sticker list {TYPE} {URI}``

            Lists the stickers for the specified object.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^sticker set "(?P<field>[^"]+)" "(?P<uri>[^"]+)" '
        r'"(?P<name>[^"]+)" "(?P<value>[^"]+)"$')
    def _sticker_set(self, field, uri, name, value):
        """
        *musicpd.org, sticker section:*

            ``sticker set {TYPE} {URI} {NAME} {VALUE}``

            Adds a sticker value to the specified object. If a sticker item
            with that name already exists, it is replaced.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^listplaylist "(?P<name>[^"]+)"$')
    def _stored_playlists_listplaylist(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``listplaylist {NAME}``

            Lists the files in the playlist ``NAME.m3u``.

        Output format::

            file: relative/path/to/file1.flac
            file: relative/path/to/file2.ogg
            file: relative/path/to/file3.mp3
        """
        try:
            return ['file: %s' % t.uri
                for t in self.backend.stored_playlists.get(name=name).tracks]
        except LookupError:
            raise MpdNoExistError(u'No such playlist', command=u'listplaylist')

    @handle_pattern(r'^listplaylistinfo "(?P<name>[^"]+)"$')
    def _stored_playlists_listplaylistinfo(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``listplaylistinfo {NAME}``

            Lists songs in the playlist ``NAME.m3u``.

        Output format:

            Standard track listing, with fields: file, Time, Title, Date,
            Album, Artist, Track
        """
        try:
            return self.backend.stored_playlists.get(name=name).mpd_format()
        except LookupError:
            raise MpdNoExistError(
                u'No such playlist', command=u'listplaylistinfo')

    @handle_pattern(r'^listplaylists$')
    def _stored_playlists_listplaylists(self):
        """
        *musicpd.org, stored playlists section:*

            ``listplaylists``

            Prints a list of the playlist directory.

            After each playlist name the server sends its last modification
            time as attribute ``Last-Modified`` in ISO 8601 format. To avoid
            problems due to clock differences between clients and the server,
            clients should not compare this value with their local clock.

        Output format::

            playlist: a
            Last-Modified: 2010-02-06T02:10:25Z
            playlist: b
            Last-Modified: 2010-02-06T02:11:08Z
        """
        result = []
        for playlist in self.backend.stored_playlists.playlists:
            result.append((u'playlist', playlist.name))
            last_modified = (playlist.last_modified or
                dt.datetime.now()).isoformat()
            # Remove microseconds
            last_modified = last_modified.split('.')[0]
            # Add time zone information
            # TODO Convert to UTC before adding Z
            last_modified = last_modified + 'Z'
            result.append((u'Last-Modified', last_modified))
        return result

    @handle_pattern(r'^load "(?P<name>[^"]+)"$')
    def _stored_playlists_load(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``load {NAME}``

            Loads the playlist ``NAME.m3u`` from the playlist directory.
        """
        matches = self.backend.stored_playlists.search(name)
        if matches:
            self.backend.current_playlist.load(matches[0].tracks)

    @handle_pattern(r'^playlistadd "(?P<name>[^"]+)" "(?P<uri>[^"]+)"$')
    def _stored_playlist_playlistadd(self, name, uri):
        """
        *musicpd.org, stored playlists section:*

            ``playlistadd {NAME} {URI}``

            Adds ``URI`` to the playlist ``NAME.m3u``.

            ``NAME.m3u`` will be created if it does not exist.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^playlistclear "(?P<name>[^"]+)"$')
    def _stored_playlist_playlistclear(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``playlistclear {NAME}``

            Clears the playlist ``NAME.m3u``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^playlistdelete "(?P<name>[^"]+)" "(?P<songpos>\d+)"$')
    def _stored_playlist_playlistdelete(self, name, songpos):
        """
        *musicpd.org, stored playlists section:*

            ``playlistdelete {NAME} {SONGPOS}``

            Deletes ``SONGPOS`` from the playlist ``NAME.m3u``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^playlistmove "(?P<name>[^"]+)" '
        r'"(?P<from_pos>\d+)" "(?P<to_pos>\d+)"$')
    def _stored_playlist_playlistmove(self, name, from_pos, to_pos):
        """
        *musicpd.org, stored playlists section:*

            ``playlistmove {NAME} {SONGID} {SONGPOS}``

            Moves ``SONGID`` in the playlist ``NAME.m3u`` to the position
            ``SONGPOS``.

        *Clarifications:*

            - The second argument is not a ``SONGID`` as used elsewhere in the
              protocol documentation, but just the ``SONGPOS`` to move *from*,
              i.e. ``playlistmove {NAME} {FROM_SONGPOS} {TO_SONGPOS}``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^rename "(?P<old_name>[^"]+)" "(?P<new_name>[^"]+)"$')
    def _stored_playlists_rename(self, old_name, new_name):
        """
        *musicpd.org, stored playlists section:*

            ``rename {NAME} {NEW_NAME}``

            Renames the playlist ``NAME.m3u`` to ``NEW_NAME.m3u``.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^rm "(?P<name>[^"]+)"$')
    def _stored_playlists_rm(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``rm {NAME}``

            Removes the playlist ``NAME.m3u`` from the playlist directory.
        """
        raise MpdNotImplemented # TODO

    @handle_pattern(r'^save "(?P<name>[^"]+)"$')
    def _stored_playlists_save(self, name):
        """
        *musicpd.org, stored playlists section:*

            ``save {NAME}``

            Saves the current playlist to ``NAME.m3u`` in the playlist
            directory.
        """
        raise MpdNotImplemented # TODO



from mopidy.mixers import BaseMixer

class DummyMixer(BaseMixer):
    """Mixer which just stores and reports the chosen volume."""

    def __init__(self, *args, **kwargs):
        super(DummyMixer, self).__init__(*args, **kwargs)
        self._volume = None

    def _get_volume(self):
        return self._volume

    def _set_volume(self, volume):
        self._volume = volume



def track_to_mpd_format(track, position=None, cpid=None):
    """
    Format track for output to MPD client.

    :param track: the track
    :type track: :class:`mopidy.models.Track`
    :param position: track's position in playlist
    :type position: integer
    :param cpid: track's CPID (current playlist ID)
    :type cpid: integer
    :rtype: list of two-tuples
    """
    result = [
        ('file', track.uri or ''),
        ('Time', track.length and (track.length // 1000) or 0),
        ('Artist', track_artists_to_mpd_format(track)),
        ('Title', track.name or ''),
        ('Album', track.album and track.album.name or ''),
        ('Date', track.date or ''),
    ]
    if track.album is not None and track.album.num_tracks != 0:
        result.append(('Track', '%d/%d' % (
            track.track_no, track.album.num_tracks)))
    else:
        result.append(('Track', track.track_no))
    if position is not None and cpid is not None:
        result.append(('Pos', position))
        result.append(('Id', cpid))
    return result

def track_artists_to_mpd_format(track):
    """
    Format track artists for output to MPD client.

    :param track: the track
    :type track: :class:`mopidy.models.Track`
    :rtype: string
    """
    artists = track.artists
    artists.sort(key=lambda a: a.name)
    return u', '.join([a.name for a in artists])

def tracks_to_mpd_format(tracks, start=0, end=None, cpids=None):
    """
    Format list of tracks for output to MPD client.

    Optionally limit output to the slice ``[start:end]`` of the list.

    :param tracks: the tracks
    :type tracks: list of :class:`mopidy.models.Track`
    :param start: position of first track to include in output
    :type start: int (positive or negative)
    :param end: position after last track to include in output
    :type end: int (positive or negative) or :class:`None` for end of list
    :rtype: list of lists of two-tuples
    """
    if end is None:
        end = len(tracks)
    tracks = tracks[start:end]
    positions = range(start, end)
    cpids = cpids and cpids[start:end] or [None for t in tracks]
    assert len(tracks) == len(positions) == len(cpids)
    result = []
    for track, position, cpid in zip(tracks, positions, cpids):
        result.append(track_to_mpd_format(track, position, cpid))
    return result

def playlist_to_mpd_format(playlist, *args, **kwargs):
    """
    Format playlist for output to MPD client.

    Arguments as for :func:`tracks_to_mpd_format`, except the first one.
    """
    return tracks_to_mpd_format(playlist.tracks, *args, **kwargs)



import datetime as dt
import logging
import os
import multiprocessing
import threading

from spotify import Link, SpotifyError
from spotify.manager import SpotifySessionManager
from spotify.alsahelper import AlsaController

from mopidy import get_version, settings
from mopidy.backends import (BaseBackend, BaseCurrentPlaylistController,
    BaseLibraryController, BasePlaybackController,
    BaseStoredPlaylistsController)
from mopidy.models import Artist, Album, Track, Playlist
from mopidy.utils import spotify_uri_to_int

import alsaaudio

logger = logging.getLogger('mopidy.backends.libspotify')

ENCODING = 'utf-8'

class LibspotifyBackend(BaseBackend):
    """
    A Spotify backend which uses the official `libspotify library
    <http://developer.spotify.com/en/libspotify/overview/>`_.

    `pyspotify <http://github.com/winjer/pyspotify/>`_ is the Python bindings
    for libspotify. It got no documentation, but multiple examples are
    available. Like libspotify, pyspotify's calls are mostly asynchronous.

    This backend should also work with `openspotify
    <http://github.com/noahwilliamsson/openspotify>`_, but we haven't tested
    that yet.

    **Issues:** http://github.com/jodal/mopidy/issues/labels/backend-libspotify
    """

    def __init__(self, *args, **kwargs):
        super(LibspotifyBackend, self).__init__(*args, **kwargs)
        self.current_playlist = LibspotifyCurrentPlaylistController(
            backend=self)
        self.library = LibspotifyLibraryController(backend=self)
        self.playback = LibspotifyPlaybackController(backend=self)
        self.stored_playlists = LibspotifyStoredPlaylistsController(
            backend=self)
        self.uri_handlers = [u'spotify:', u'http://open.spotify.com/']
        self.audio_controller_class = kwargs.get(
            'audio_controller_class', AlsaController)
        self.spotify = self._connect()

    def _connect(self):
        logger.info(u'Connecting to Spotify')
        spotify = LibspotifySessionManager(
            settings.SPOTIFY_USERNAME, settings.SPOTIFY_PASSWORD,
            core_queue=self.core_queue,
            audio_controller_class=self.audio_controller_class)
        spotify.start()
        return spotify


class LibspotifyCurrentPlaylistController(BaseCurrentPlaylistController):
    pass


class LibspotifyLibraryController(BaseLibraryController):
    def lookup(self, uri):
        spotify_track = Link.from_string(uri).as_track()
        return LibspotifyTranslator.to_mopidy_track(spotify_track)

    def search(self, field, what):
        if field is u'any':
            query = what
        else:
            query = u'%s:%s' % (field, what)
        my_end, other_end = multiprocessing.Pipe()
        self.backend.spotify.search(query.encode(ENCODING), other_end)
        my_end.poll(None)
        logger.debug(u'In search method, receiving search results')
        playlist = my_end.recv()
        logger.debug(u'In search method, done receiving search results')
        logger.debug(['%s' % t.name for t in playlist.tracks])
        return playlist

    find_exact = search


class LibspotifyPlaybackController(BasePlaybackController):
    def _pause(self):
        # TODO
        return False

    def _play(self, track):
        if self.state == self.PLAYING:
            self.stop()
        if track.uri is None:
            return False
        try:
            self.backend.spotify.session.load(
                Link.from_string(track.uri).as_track())
            self.backend.spotify.session.play(1)
            return True
        except SpotifyError as e:
            logger.warning('Play %s failed: %s', track.uri, e)
            return False

    def _resume(self):
        # TODO
        return False

    def _stop(self):
        self.backend.spotify.session.play(0)
        return True


class LibspotifyStoredPlaylistsController(BaseStoredPlaylistsController):
    pass


class LibspotifyTranslator(object):
    @classmethod
    def to_mopidy_id(cls, spotify_uri):
        return spotify_uri_to_int(spotify_uri)

    @classmethod
    def to_mopidy_artist(cls, spotify_artist):
        if not spotify_artist.is_loaded():
            return Artist(name=u'[loading...]')
        return Artist(
            uri=str(Link.from_artist(spotify_artist)),
            name=spotify_artist.name().decode(ENCODING),
        )

    @classmethod
    def to_mopidy_album(cls, spotify_album):
        if not spotify_album.is_loaded():
            return Album(name=u'[loading...]')
        # TODO pyspotify got much more data on albums than this
        return Album(name=spotify_album.name().decode(ENCODING))

    @classmethod
    def to_mopidy_track(cls, spotify_track):
        if not spotify_track.is_loaded():
            return Track(name=u'[loading...]')
        uri = str(Link.from_track(spotify_track, 0))
        if dt.MINYEAR <= int(spotify_track.album().year()) <= dt.MAXYEAR:
            date = dt.date(spotify_track.album().year(), 1, 1)
        else:
            date = None
        return Track(
            uri=uri,
            name=spotify_track.name().decode(ENCODING),
            artists=[cls.to_mopidy_artist(a) for a in spotify_track.artists()],
            album=cls.to_mopidy_album(spotify_track.album()),
            track_no=spotify_track.index(),
            date=date,
            length=spotify_track.duration(),
            bitrate=320,
            id=cls.to_mopidy_id(uri),
        )

    @classmethod
    def to_mopidy_playlist(cls, spotify_playlist):
        if not spotify_playlist.is_loaded():
            return Playlist(name=u'[loading...]')
        return Playlist(
            uri=str(Link.from_playlist(spotify_playlist)),
            name=spotify_playlist.name().decode(ENCODING),
            tracks=[cls.to_mopidy_track(t) for t in spotify_playlist],
        )


class LibspotifySessionManager(SpotifySessionManager, threading.Thread):
    cache_location = os.path.expanduser(settings.SPOTIFY_LIB_CACHE)
    settings_location = os.path.expanduser(settings.SPOTIFY_LIB_CACHE)
    appkey_file = os.path.expanduser(settings.SPOTIFY_LIB_APPKEY)
    user_agent = 'Mopidy %s' % get_version()

    def __init__(self, username, password, core_queue, audio_controller_class):
        SpotifySessionManager.__init__(self, username, password)
        threading.Thread.__init__(self)
        self.core_queue = core_queue
        self.connected = threading.Event()
        self.audio = audio_controller_class(alsaaudio.PCM_NORMAL)
        self.session = None

    def run(self):
        self.connect()

    def logged_in(self, session, error):
        """Callback used by pyspotify"""
        logger.info('Logged in')
        self.session = session
        self.connected.set()

    def logged_out(self, session):
        """Callback used by pyspotify"""
        logger.info('Logged out')

    def metadata_updated(self, session):
        """Callback used by pyspotify"""
        logger.debug('Metadata updated, refreshing stored playlists')
        playlists = []
        for spotify_playlist in session.playlist_container():
            playlists.append(
                LibspotifyTranslator.to_mopidy_playlist(spotify_playlist))
        self.core_queue.put({
            'command': 'set_stored_playlists',
            'playlists': playlists,
        })

    def connection_error(self, session, error):
        """Callback used by pyspotify"""
        logger.error('Connection error: %s', error)

    def message_to_user(self, session, message):
        """Callback used by pyspotify"""
        logger.info(message)

    def notify_main_thread(self, session):
        """Callback used by pyspotify"""
        logger.debug('Notify main thread')

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        """Callback used by pyspotify"""
        self.audio.music_delivery(session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels)

    def play_token_lost(self, session):
        """Callback used by pyspotify"""
        logger.debug('Play token lost')
        self.core_queue.put({'command': 'stop_playback'})

    def log_message(self, session, data):
        """Callback used by pyspotify"""
        logger.debug(data)

    def end_of_track(self, session):
        """Callback used by pyspotify"""
        logger.debug('End of track')
        self.core_queue.put({'command': 'end_of_track'})

    def search(self, query, connection):
        """Search method used by Mopidy backend"""
        self.connected.wait()
        def callback(results, userdata):
            logger.debug(u'In search callback, translating search results')
            logger.debug(results.tracks())
            # TODO Include results from results.albums(), etc. too
            playlist = Playlist(tracks=[
                LibspotifyTranslator.to_mopidy_track(t)
                for t in results.tracks()])
            logger.debug(u'In search callback, sending search results')
            logger.debug(['%s' % t.name for t in playlist.tracks])
            connection.send(playlist)
        self.session.search(query, callback)
